#!/usr/bin/env python
"""Test file for the physbiblio.gui.dialogWindows module.

This file is part of the physbiblio package.
"""
import sys
import traceback
import os
from PySide2.QtCore import Qt, QEvent, QModelIndex, QPoint, QRect
from PySide2.QtTest import QTest
from PySide2.QtWidgets import QWidget

if sys.version_info[0] < 3:
    import unittest2 as unittest
    from mock import patch, call, MagicMock
else:
    import unittest
    from unittest.mock import patch, call, MagicMock

try:
    from physbiblio.setuptests import *
    from physbiblio.database import pBDB
    from physbiblio.config import pbConfig, configuration_params
    from physbiblio.gui.setuptests import *
    from physbiblio.gui.dialogWindows import *
    from physbiblio.gui.bibWindows import AbstractFormulas
except ImportError:
    print("Could not find physbiblio and its modules!")
    print(traceback.format_exc())
    raise


class Fake_abstractFormulas:
    """Used to test `DailyArxivSelect.cellClick`"""

    def __init__(self):
        """empty constructor"""
        return

    def __call__(self, p, abstract, customEditor=None, statusMessages=True):
        """Save some attributes and return self.el

        Parameters: (see also `physbiblio.gui.bibWindows.AbstractFormulas`)
            p: parent widget
            abstract: the abstract
            customEditor: widget where to store the processed text
            statusMessages: print status messages
        """
        self.par = p
        self.abstract = abstract
        self.ce = customEditor
        self.sm = statusMessages
        self.el = AbstractFormulas(
            p, abstract, customEditor=self.ce, statusMessages=self.sm
        )
        return self.el


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestConfigEditColumns(GUITestCase):
    """Test ConfigEditColumns"""

    @classmethod
    def setUpClass(self):
        """set temporary settings"""
        super(TestConfigEditColumns, self).setUpClass()
        self.defCols = [
            a["default"]
            for a in configuration_params.values()
            if a["name"] == "bibtexListColumns"
        ][0]

    @patch.dict(
        pbConfig.params,
        {
            "bibtexListColumns": [
                a["default"]
                for a in configuration_params.values()
                if a["name"] == "bibtexListColumns"
            ][0]
        },
        clear=False,
    )
    def test_init(self):
        """Test __init__"""
        p = QWidget()
        with patch(
            "physbiblio.gui.dialogWindows.ConfigEditColumns.initUI", autospec=True
        ) as _u:
            cec = ConfigEditColumns()
            self.assertIsInstance(cec, PBDialog)
            self.assertEqual(cec.parent(), None)
            _u.assert_called_once_with(cec)
            cec = ConfigEditColumns(p)
            self.assertIsInstance(cec, PBDialog)
            self.assertEqual(cec.parent(), p)
            self.assertEqual(
                cec.excludeCols,
                [
                    "crossref",
                    "bibtex",
                    "exp_paper",
                    "lecture",
                    "phd_thesis",
                    "review",
                    "proceeding",
                    "book",
                    "noUpdate",
                    "bibdict",
                    "abstract",
                ],
            )
            self.assertEqual(
                cec.moreCols,
                [
                    "title",
                    "author",
                    "journal",
                    "volume",
                    "pages",
                    "primaryclass",
                    "booktitle",
                    "reportnumber",
                ],
            )
            self.assertEqual(cec.previousSelected, self.defCols)
            self.assertEqual(cec.selected, self.defCols)
            cec = ConfigEditColumns(p, ["bibkey", "author", "title"])
            self.assertEqual(cec.previousSelected, ["bibkey", "author", "title"])

    @patch.dict(
        pbConfig.params,
        {
            "bibtexListColumns": [
                a["default"]
                for a in configuration_params.values()
                if a["name"] == "bibtexListColumns"
            ][0]
        },
        clear=False,
    )
    def test_onCancel(self):
        """test onCancel"""
        cec = ConfigEditColumns()
        with patch("PySide2.QtWidgets.QDialog.close", autospec=True) as _c:
            cec.onCancel()
            self.assertFalse(cec.result)
            self.assertEqual(_c.call_count, 1)

    @patch.dict(
        pbConfig.params,
        {
            "bibtexListColumns": [
                a["default"]
                for a in configuration_params.values()
                if a["name"] == "bibtexListColumns"
            ][0]
        },
        clear=False,
    )
    def test_onOk(self):
        """test onOk"""
        p = QWidget()
        cec = ConfigEditColumns(p, ["bibkey", "author", "title"])
        with patch("PySide2.QtWidgets.QDialog.close", autospec=True) as _c:
            cec.onOk()
            self.assertTrue(cec.result)
            self.assertEqual(_c.call_count, 1)
        self.assertEqual(cec.selected, ["bibkey", "author", "title"])
        item = QTableWidgetItem("arxiv")
        cec.listSel.insertRow(3)
        cec.listSel.setItem(3, 0, item)
        with patch("PySide2.QtWidgets.QDialog.close", autospec=True) as _c:
            cec.onOk()
        self.assertEqual(cec.selected, ["bibkey", "author", "title", "arxiv"])

    @patch.dict(
        pbConfig.params,
        {
            "bibtexListColumns": [
                a["default"]
                for a in configuration_params.values()
                if a["name"] == "bibtexListColumns"
            ][0]
        },
        clear=False,
    )
    def test_initUI(self):
        """test initUI"""
        p = QWidget()
        cec = ConfigEditColumns(p, ["bibkey", "author", "title"])
        self.assertIsInstance(cec.layout(), QGridLayout)
        self.assertEqual(cec.layout(), cec.gridlayout)
        self.assertIsInstance(cec.items, list)
        self.assertIsInstance(cec.allItems, list)
        self.assertIsInstance(cec.selItems, list)
        self.assertIsInstance(cec.listAll, PBDDTableWidget)
        self.assertIsInstance(cec.listSel, PBDDTableWidget)
        self.assertIsInstance(cec.layout().itemAtPosition(0, 0).widget(), PBLabel)
        self.assertEqual(
            cec.layout().itemAtPosition(0, 0).widget().text(),
            "Drag and drop items to order visible columns",
        )
        self.assertEqual(cec.layout().itemAtPosition(1, 0).widget(), cec.listAll)
        self.assertEqual(cec.layout().itemAtPosition(1, 1).widget(), cec.listSel)
        self.assertEqual(
            cec.allItems, list(pBDB.descriptions["entries"]) + cec.moreCols
        )
        self.assertEqual(cec.selItems, cec.previousSelected)

        self.assertEqual(cec.listSel.rowCount(), 3)
        for ix, col in enumerate(["bibkey", "author", "title"]):
            item = cec.listSel.item(ix, 0)
            self.assertEqual(item.text(), col)
            self.assertIs(item, cec.items[ix])

        allCols = [
            i
            for i in cec.allItems
            if i not in cec.selItems and i not in cec.excludeCols
        ]
        self.assertEqual(cec.listAll.rowCount(), len(allCols))
        for ix, col in enumerate(allCols):
            item = cec.listAll.item(ix, 0)
            self.assertEqual(item.text(), col)
            self.assertIs(item, cec.items[ix + 3])

        self.assertIsInstance(cec.acceptButton, QPushButton)
        self.assertIsInstance(cec.cancelButton, QPushButton)
        self.assertEqual(cec.acceptButton.text(), "OK")
        self.assertEqual(cec.cancelButton.text(), "Cancel")
        self.assertTrue(cec.cancelButton.autoDefault())
        self.assertEqual(cec.layout().itemAtPosition(2, 0).widget(), cec.acceptButton)
        self.assertEqual(cec.layout().itemAtPosition(2, 1).widget(), cec.cancelButton)
        with patch(
            "physbiblio.gui.dialogWindows.ConfigEditColumns.onOk", autospec=True
        ) as _f:
            QTest.mouseClick(cec.acceptButton, Qt.LeftButton)
            _f.assert_called_once_with(cec)
        with patch(
            "physbiblio.gui.dialogWindows.ConfigEditColumns.onCancel", autospec=True
        ) as _f:
            QTest.mouseClick(cec.cancelButton, Qt.LeftButton)
            _f.assert_called_once_with(cec)


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestConfigWindow(GUITestCase):
    """Test ConfigWindow"""

    def test_init(self):
        """Test __init__"""
        p = QWidget()
        with patch(
            "physbiblio.gui.dialogWindows.ConfigWindow.initUI", autospec=True
        ) as _iu:
            cw = ConfigWindow(p)
            self.assertIsInstance(cw, PBDialog)
            self.assertEqual(cw.parent(), p)
            self.assertEqual(cw.textValues, [])
            _iu.assert_called_once_with(cw)

    def test_onCancel(self):
        """test onCancel"""
        cw = ConfigWindow()
        with patch("PySide2.QtWidgets.QDialog.close", autospec=True) as _c:
            cw.onCancel()
            self.assertFalse(cw.result)
            self.assertEqual(_c.call_count, 1)

    def test_onOk(self):
        """test onOk"""
        cw = ConfigWindow()
        with patch("PySide2.QtWidgets.QDialog.close", autospec=True) as _c:
            cw.onOk()
            self.assertTrue(cw.result)
            self.assertEqual(_c.call_count, 1)

    def test_editPDFFolder(self):
        """test editPDFFolder"""
        cw = ConfigWindow()
        ix = pbConfig.paramOrder.index("pdfFolder")
        self.assertEqual(cw.textValues[ix][1].text(), pbConfig.params["pdfFolder"])
        with patch(
            "physbiblio.gui.dialogWindows.askDirName",
            return_value="/some/new/folder/",
            autospec=True,
        ) as _adn:
            cw.editPDFFolder()
            self.assertEqual(cw.textValues[ix][1].text(), "/some/new/folder/")
            _adn.assert_called_once_with(
                parent=None,
                dir=pbConfig.params["pdfFolder"],
                title="Directory for saving PDF files:",
            )
        with patch(
            "physbiblio.gui.dialogWindows.askDirName", return_value="", autospec=True
        ) as _adn:
            cw.editPDFFolder()
            self.assertEqual(cw.textValues[ix][1].text(), "/some/new/folder/")
            _adn.assert_called_once_with(
                parent=None,
                dir="/some/new/folder/",
                title="Directory for saving PDF files:",
            )

    def test_editFile(self):
        """test editFile"""
        cw = ConfigWindow()
        ix = pbConfig.paramOrder.index("logFileName")
        self.assertEqual(cw.textValues[ix][1].text(), pbConfig.params["logFileName"])
        with patch(
            "physbiblio.gui.dialogWindows.askSaveFileName",
            return_value="/some/new/folder/file.log",
            autospec=True,
        ) as _adn:
            cw.editFile()
            self.assertEqual(cw.textValues[ix][1].text(), "/some/new/folder/file.log")
            _adn.assert_called_once_with(
                parent=None,
                title="Name for the log file",
                dir=pbConfig.params["logFileName"],
                filter="*.log",
            )
        with patch(
            "physbiblio.gui.dialogWindows.askSaveFileName",
            return_value="  ",
            autospec=True,
        ) as _adn:
            cw.editFile(text="Name", filter="*.txt")
            self.assertEqual(cw.textValues[ix][1].text(), "/some/new/folder/file.log")
            _adn.assert_called_once_with(
                parent=None,
                title="Name",
                dir="/some/new/folder/file.log",
                filter="*.txt",
            )
        with patch("logging.Logger.warning") as _w:
            cw.editFile("someField")
            _w.assert_called_once_with("Invalid paramkey: 'someField'")

    def test_editColumns(self):
        """test editColumns"""
        cw = ConfigWindow()
        ix = pbConfig.paramOrder.index("bibtexListColumns")
        self.assertEqual(
            ast.literal_eval(cw.textValues[ix][1].text()),
            pbConfig.params["bibtexListColumns"],
        )
        cec = ConfigEditColumns(cw, ["bibkey", "author", "title"])
        cec.exec_ = MagicMock()
        cec.onCancel()
        with patch(
            "physbiblio.gui.dialogWindows.ConfigEditColumns",
            return_value=cec,
            autospec=USE_AUTOSPEC_CLASS,
        ) as _cec:
            cw.editColumns()
            _cec.assert_called_once_with(cw, pbConfig.params["bibtexListColumns"])
        self.assertEqual(
            ast.literal_eval(cw.textValues[ix][1].text()),
            pbConfig.params["bibtexListColumns"],
        )
        cec = ConfigEditColumns(cw, ["bibkey", "author", "title"])
        cec.exec_ = MagicMock()
        cec.onOk()
        with patch(
            "physbiblio.gui.dialogWindows.ConfigEditColumns",
            return_value=cec,
            autospec=USE_AUTOSPEC_CLASS,
        ) as _cec:
            cw.editColumns()
            _cec.assert_called_once_with(cw, pbConfig.params["bibtexListColumns"])
        self.assertEqual(
            ast.literal_eval(cw.textValues[ix][1].text()), ["bibkey", "author", "title"]
        )

    def test_editDefCats(self):
        """test editDefCats"""
        cw = ConfigWindow()
        ix = pbConfig.paramOrder.index("defaultCategories")
        self.assertEqual(
            ast.literal_eval(cw.textValues[ix][1].text()),
            pbConfig.params["defaultCategories"],
        )
        cwl = CatsTreeWindow(parent=cw, askCats=True, expButton=False, previous=["1"])
        cwl.exec_ = MagicMock()
        cwl.onCancel()
        with patch(
            "physbiblio.gui.dialogWindows.CatsTreeWindow",
            return_value=cwl,
            autospec=USE_AUTOSPEC_CLASS,
        ) as _cwl:
            cw.editDefCats()
            _cwl.assert_called_once_with(
                parent=cw,
                askCats=True,
                expButton=False,
                previous=pbConfig.params["defaultCategories"],
            )
        self.assertEqual(
            ast.literal_eval(cw.textValues[ix][1].text()),
            pbConfig.params["defaultCategories"],
        )
        cwl = CatsTreeWindow(parent=cw, askCats=True, expButton=False, previous=["1"])
        cwl.exec_ = MagicMock()
        cwl.onOk()
        cw.selectedCats = ["1"]
        with patch(
            "physbiblio.gui.dialogWindows.CatsTreeWindow",
            return_value=cwl,
            autospec=USE_AUTOSPEC_CLASS,
        ) as _cwl:
            cw.editDefCats()
            _cwl.assert_called_once_with(
                parent=cw,
                askCats=True,
                expButton=False,
                previous=pbConfig.params["defaultCategories"],
            )
        self.assertEqual(ast.literal_eval(cw.textValues[ix][1].text()), ["1"])

    def test_initUI(self):
        """test initUI"""
        cw = ConfigWindow()
        self.assertEqual(cw.windowTitle(), "Configuration")
        self.assertIsInstance(cw.layout(), QGridLayout)
        self.assertEqual(cw.layout(), cw.grid)
        self.assertEqual(cw.grid.spacing(), 1)

        for ix, k in enumerate(pbConfig.paramOrder):
            self.assertIsInstance(cw.layout().itemAtPosition(ix, 0).widget(), PBLabel)
            self.assertEqual(
                cw.layout().itemAtPosition(ix, 0).widget().text(),
                "%s (<i>%s</i>%s)"
                % (
                    configuration_params[k].description,
                    k,
                    " - global setting" if configuration_params[k].isGlobal else "",
                ),
            )
            if k == "bibtexListColumns":
                currClass = QPushButton
                currWidget = cw.textValues[ix][1]
                self.assertIsInstance(currWidget, currClass)
                self.assertEqual(currWidget.text(), "%s" % pbConfig.params[k])
                with patch(
                    "physbiblio.gui.dialogWindows.ConfigWindow.editColumns",
                    autospec=True,
                ) as _f:
                    QTest.mouseClick(currWidget, Qt.LeftButton)
                    _f.assert_called_once_with(cw)
            elif k == "pdfFolder":
                currClass = QPushButton
                currWidget = cw.textValues[ix][1]
                self.assertIsInstance(currWidget, currClass)
                self.assertEqual(currWidget.text(), "%s" % pbConfig.params[k])
                with patch(
                    "physbiblio.gui.dialogWindows.ConfigWindow.editPDFFolder",
                    autospec=True,
                ) as _f:
                    QTest.mouseClick(currWidget, Qt.LeftButton)
                    _f.assert_called_once_with(cw)
            elif k == "loggingLevel":
                currClass = PBComboBox
                currWidget = cw.textValues[ix][1]
                self.assertIsInstance(currWidget, currClass)
                self.assertEqual(
                    currWidget.currentText(),
                    pbConfig.loggingLevels[int(pbConfig.params[k])],
                )
                self.assertEqual(currWidget.count(), len(pbConfig.loggingLevels))
                for j, l in enumerate(pbConfig.loggingLevels):
                    self.assertEqual(currWidget.itemText(j), pbConfig.loggingLevels[j])
            elif k == "logFileName":
                currClass = QPushButton
                currWidget = cw.textValues[ix][1]
                self.assertIsInstance(currWidget, currClass)
                self.assertEqual(currWidget.text(), "%s" % pbConfig.params[k])
                with patch(
                    "physbiblio.gui.dialogWindows.ConfigWindow.editFile", autospec=True
                ) as _f:
                    QTest.mouseClick(currWidget, Qt.LeftButton)
                    _f.assert_called_once_with(cw)
            elif k == "defaultCategories":
                currClass = QPushButton
                currWidget = cw.textValues[ix][1]
                self.assertIsInstance(currWidget, currClass)
                self.assertEqual(currWidget.text(), "%s" % pbConfig.params[k])
                with patch(
                    "physbiblio.gui.dialogWindows.ConfigWindow.editDefCats",
                    autospec=True,
                ) as _f:
                    QTest.mouseClick(currWidget, Qt.LeftButton)
                    _f.assert_called_once_with(cw)
            elif configuration_params[k].special == "boolean":
                currClass = PBTrueFalseCombo
                currWidget = cw.textValues[ix][1]
                self.assertIsInstance(currWidget, currClass)
                self.assertEqual(currWidget.currentText(), "%s" % pbConfig.params[k])
            else:
                currClass = QLineEdit
                currWidget = cw.textValues[ix][1]
                self.assertIsInstance(currWidget, currClass)
                self.assertEqual(currWidget.text(), "%s" % pbConfig.params[k])
            self.assertIsInstance(cw.layout().itemAtPosition(ix, 2).widget(), currClass)
            self.assertEqual(cw.layout().itemAtPosition(ix, 2).widget(), currWidget)

        self.assertIsInstance(cw.acceptButton, QPushButton)
        self.assertIsInstance(cw.cancelButton, QPushButton)
        self.assertEqual(cw.acceptButton.text(), "OK")
        self.assertEqual(cw.cancelButton.text(), "Cancel")
        self.assertTrue(cw.cancelButton.autoDefault())
        ix = len(pbConfig.paramOrder)
        self.assertEqual(cw.layout().itemAtPosition(ix, 0).widget(), cw.acceptButton)
        self.assertEqual(cw.layout().itemAtPosition(ix, 1).widget(), cw.cancelButton)
        with patch(
            "physbiblio.gui.dialogWindows.ConfigWindow.onOk", autospec=True
        ) as _f:
            QTest.mouseClick(cw.acceptButton, Qt.LeftButton)
            _f.assert_called_once_with(cw)
        with patch(
            "physbiblio.gui.dialogWindows.ConfigWindow.onCancel", autospec=True
        ) as _f:
            QTest.mouseClick(cw.cancelButton, Qt.LeftButton)
            _f.assert_called_once_with(cw)

        # test non valid value for loggingLevel
        with patch("logging.Logger.warning") as _w, patch.dict(
            pbConfig.params, {"loggingLevel": "10"}, clear=False
        ):
            cw = ConfigWindow()
            _w.assert_called_once_with(
                "Invalid string for 'loggingLevel' param. Reset to default"
            )
        with patch("logging.Logger.warning") as _w, patch.dict(
            pbConfig.params, {"loggingLevel": "abc"}, clear=False
        ):
            cw = ConfigWindow()
            _w.assert_called_once_with(
                "Invalid string for 'loggingLevel' param. Reset to default"
            )
        for ix, k in enumerate(pbConfig.paramOrder):
            if k == "loggingLevel":
                currClass = PBComboBox
                currWidget = cw.textValues[ix][1]
                self.assertIsInstance(currWidget, currClass)
                self.assertEqual(
                    currWidget.currentText(),
                    pbConfig.loggingLevels[
                        int(configuration_params["loggingLevel"].default)
                    ],
                )
                self.assertEqual(currWidget.count(), len(pbConfig.loggingLevels))
                for j, l in enumerate(pbConfig.loggingLevels):
                    self.assertEqual(currWidget.itemText(j), pbConfig.loggingLevels[j])


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestLogFileContentDialog(GUITestCase):
    """Test LogFileContentDialog"""

    def test_init(self):
        """test __init__"""
        p = QWidget()
        with patch(
            "physbiblio.gui.dialogWindows.LogFileContentDialog.initUI", autospec=True
        ) as _u:
            lf = LogFileContentDialog(p)
            self.assertIsInstance(lf, PBDialog)
            self.assertEqual(lf.parent(), p)
            self.assertEqual(lf.title, "Log File Content")
            _u.assert_called_once_with(lf)
        if os.path.exists(pbConfig.params["logFileName"]):
            os.remove(pbConfig.params["logFileName"])

    def test_clearLog(self):
        """test clearLog"""
        p = QWidget()
        with open(pbConfig.params["logFileName"], "w") as _f:
            _f.write("test content")
        lf = LogFileContentDialog(p)
        ayn_str = "physbiblio.gui.dialogWindows.askYesNo"
        with patch(ayn_str, return_value=False, autospec=True) as _ayn:
            lf.clearLog()
            with open(pbConfig.params["logFileName"]) as _f:
                text = _f.read()
            self.assertEqual(text, "test content")
        with patch(ayn_str, return_value=True, autospec=True) as _ayn, patch(
            "physbiblio.gui.dialogWindows.infoMessage", autospec=True
        ) as _in, patch("PySide2.QtWidgets.QDialog.close", autospec=True) as _c:
            lf.clearLog()
            with open(pbConfig.params["logFileName"]) as _f:
                text = _f.read()
            self.assertEqual(text, "")
            _in.assert_called_once_with("Log file cleared.")
            _c.assert_called_once_with()
        if os.path.exists(pbConfig.params["logFileName"]):
            os.remove(pbConfig.params["logFileName"])
        openModule = "__builtin__.open" if sys.version_info[0] < 3 else "builtins.open"
        with patch(ayn_str, return_value=True, autospec=True) as _ayn, patch(
            openModule, side_effect=IOError("fake"), autospec=True
        ) as _op, patch("logging.Logger.exception") as _ex, patch(
            "PySide2.QtWidgets.QDialog.close", autospec=True
        ) as _c:
            lf.clearLog()
            _ex.assert_called_once_with("Impossible to clear log file!")
            _c.assert_not_called()
        if os.path.exists(pbConfig.params["logFileName"]):
            os.remove(pbConfig.params["logFileName"])

    def test_initUI(self):
        """test initUI"""
        p = QWidget()
        if os.path.exists(pbConfig.params["logFileName"]):
            os.remove(pbConfig.params["logFileName"])
        with patch("logging.Logger.exception") as _ex:
            lf = LogFileContentDialog(p)
            _ex.assert_called_once_with("Impossible to read log file!")
            self.assertIsInstance(lf.textEdit, QPlainTextEdit)
            self.assertEqual(lf.textEdit.toPlainText(), "Impossible to read log file!")
        with open(pbConfig.params["logFileName"], "w") as _f:
            _f.write("test content")
        lf = LogFileContentDialog(p)
        self.assertEqual(lf.windowTitle(), lf.title)
        self.assertIsInstance(lf.layout(), QVBoxLayout)
        self.assertEqual(lf.layout().spacing(), 1)
        self.assertIsInstance(lf.layout().itemAt(0).widget(), PBLabel)
        self.assertEqual(
            lf.layout().itemAt(0).widget().text(),
            "Reading %s" % pbConfig.params["logFileName"],
        )
        self.assertIsInstance(lf.textEdit, QPlainTextEdit)
        self.assertTrue(lf.textEdit.isReadOnly())
        self.assertEqual(lf.textEdit.toPlainText(), "test content")
        self.assertIsInstance(lf.layout().itemAt(1).widget(), QPlainTextEdit)
        self.assertEqual(lf.textEdit, lf.layout().itemAt(1).widget())
        self.assertIsInstance(lf.layout().itemAt(2).widget(), QPushButton)
        self.assertEqual(lf.layout().itemAt(2).widget(), lf.closeButton)
        self.assertTrue(lf.closeButton.autoDefault())
        self.assertEqual(lf.closeButton.text(), "Close")
        self.assertIsInstance(lf.layout().itemAt(3).widget(), QPushButton)
        self.assertEqual(lf.layout().itemAt(3).widget(), lf.clearButton)
        self.assertEqual(lf.clearButton.text(), "Clear log file")
        if os.path.exists(pbConfig.params["logFileName"]):
            os.remove(pbConfig.params["logFileName"])


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestPrintText(GUITestCase):
    """Test PrintText"""

    def test_init(self):
        """test init"""
        with patch(
            "physbiblio.gui.dialogWindows.PrintText.initUI", autospec=True
        ) as _u:
            pt = PrintText()
            _u.assert_called_once_with(pt)
        self.assertIsInstance(pt, PBDialog)
        self.assertIsInstance(pt.stopped, Signal)
        self.assertIsInstance(pt.pbMax, Signal)
        self.assertIsInstance(pt.pbVal, Signal)
        self.assertEqual(pt._wantToClose, False)
        self.assertEqual(pt.parent(), None)
        self.assertEqual(pt.title, "Redirect print")
        self.assertEqual(pt.setProgressBar, True)
        self.assertEqual(pt.noStopButton, False)
        self.assertEqual(pt.message, None)
        with patch(
            "physbiblio.gui.dialogWindows.PrintText.progressBarMax", autospec=True
        ) as _m, patch(
            "physbiblio.gui.dialogWindows.PrintText.progressBarValue", autospec=True
        ) as _v, patch(
            "physbiblio.gui.dialogWindows.PrintText.initUI", autospec=True
        ) as _u:
            pt = PrintText()
            _u.assert_called_once_with(pt)
            pt.pbMax.emit(1)
            _m.assert_called_once_with(pt, 1)
            pt.pbVal.emit(2)
            _v.assert_called_once_with(pt, 2)

        p = QWidget()
        pt = PrintText(p, "title", False, True, "mymessage")
        self.assertEqual(pt.parent(), p)
        self.assertEqual(pt.title, "title")
        self.assertEqual(pt.setProgressBar, False)
        self.assertEqual(pt.noStopButton, True)
        self.assertEqual(pt.message, "mymessage")

    def test_keyPressEvent(self):
        """test keyPressEvent"""
        p = QWidget()
        pt = PrintText(p)
        with patch("PySide2.QtWidgets.QDialog.close", autospec=True) as _oc:
            QTest.keyPress(pt, "a")
            _oc.assert_not_called()
            QTest.keyPress(pt, Qt.Key_Enter)
            _oc.assert_not_called()
            QTest.keyPress(pt, Qt.Key_Escape)
            _oc.assert_not_called()
            pt.enableClose()
            QTest.keyPress(pt, Qt.Key_Escape)
            self.assertEqual(_oc.call_count, 1)

    def test_closeEvent(self):
        """test closeEvent"""
        p = QWidget()
        pt = PrintText(p)
        e = QEvent(QEvent.Close)
        with patch("PySide2.QtCore.QEvent.ignore", autospec=True) as _i:
            pt.closeEvent(e)
            _i.assert_called_once_with()
        pt._wantToClose = True
        with patch("PySide2.QtWidgets.QDialog.closeEvent", autospec=True) as _c:
            pt.closeEvent(e)
            _c.assert_called_once_with(e)

    def test_initUI(self):
        """test initUI"""
        p = QWidget()
        pt = PrintText(p, progressBar=False, noStopButton=True)
        self.assertEqual(pt.windowTitle(), "Redirect print")
        self.assertIsInstance(pt.layout(), QGridLayout)
        self.assertEqual(pt.layout(), pt.grid)
        self.assertEqual(pt.grid.spacing(), 1)
        self.assertIsInstance(pt.grid.itemAtPosition(0, 0).widget(), QTextEdit)
        self.assertEqual(pt.grid.itemAtPosition(0, 0).widget(), pt.textEdit)
        self.assertEqual(pt.textEdit.toPlainText(), "")
        self.assertIsInstance(pt.grid.itemAtPosition(1, 0).widget(), QPushButton)
        self.assertEqual(pt.grid.itemAtPosition(1, 0).widget(), pt.closeButton)
        self.assertEqual(pt.closeButton.text(), "Close")
        self.assertFalse(pt.closeButton.isEnabled())

        pt = PrintText(p, "title", True, False, "mymessage")
        self.assertIsInstance(pt.grid.itemAtPosition(0, 0).widget(), PBLabel)
        self.assertEqual(pt.grid.itemAtPosition(0, 0).widget().text(), pt.message)
        self.assertIsInstance(pt.grid.itemAtPosition(1, 0).widget(), QTextEdit)
        self.assertEqual(pt.grid.itemAtPosition(1, 0).widget(), pt.textEdit)
        self.assertEqual(pt.textEdit.toPlainText(), "")
        self.assertIsInstance(pt.grid.itemAtPosition(2, 0).widget(), QProgressBar)
        self.assertEqual(pt.grid.itemAtPosition(2, 0).widget(), pt.progressBar)
        self.assertEqual(pt.progressBar.value(), -1)
        self.assertEqual(pt.progressBar.minimum(), 0)
        self.assertEqual(pt.progressBar.maximum(), 100)
        self.assertIsInstance(pt.grid.itemAtPosition(3, 0).widget(), QPushButton)
        self.assertEqual(pt.grid.itemAtPosition(3, 0).widget(), pt.cancelButton)
        self.assertEqual(pt.cancelButton.text(), "Stop")
        self.assertTrue(pt.cancelButton.autoDefault())
        with patch(
            "physbiblio.gui.dialogWindows.PrintText.stopExec", autospec=True
        ) as _s:
            QTest.mouseClick(pt.cancelButton, Qt.LeftButton)
            _s.assert_called_once_with(pt)
        self.assertIsInstance(pt.grid.itemAtPosition(4, 0).widget(), QPushButton)
        self.assertEqual(pt.grid.itemAtPosition(4, 0).widget(), pt.closeButton)
        self.assertEqual(pt.closeButton.text(), "Close")
        self.assertFalse(pt.closeButton.isEnabled())
        with patch("PySide2.QtWidgets.QDialog.reject", autospec=True) as _s:
            QTest.mouseClick(pt.closeButton, Qt.LeftButton)
            _s.assert_not_called()
            pt.enableClose()
            self.assertTrue(pt.closeButton.isEnabled())

        with patch(
            "physbiblio.gui.dialogWindows.PrintText.centerWindow", autospec=True
        ) as _c:
            pt.initUI()
            _c.assert_called_once_with(pt)

    def test_appendText(self):
        """test appendText"""
        p = QWidget()
        pt = PrintText(p)
        self.assertEqual(pt.textEdit.toPlainText(), "")
        pt.appendText("abcd")
        self.assertEqual(pt.textEdit.toPlainText(), "abcd")
        pt.textEdit.moveCursor(QTextCursor.Start)
        pt.appendText("\nefgh")
        self.assertEqual(pt.textEdit.toPlainText(), "abcd\nefgh")

    def test_progressBarMin(self):
        """test progressBarMin"""
        pt = PrintText()
        self.assertIsInstance(pt.progressBar, QProgressBar)
        pt.progressBarMin(1234.0)
        self.assertEqual(pt.progressBar.minimum(), 1234.0)

    def test_progressBarMax(self):
        """test progressBarMax"""
        pt = PrintText()
        self.assertIsInstance(pt.progressBar, QProgressBar)
        pt.progressBarMax(1234)
        self.assertEqual(pt.progressBar.maximum(), 1234)

    def test_progressBarValue(self):
        """test progressBarValue"""
        pt = PrintText()
        self.assertIsInstance(pt.progressBar, QProgressBar)
        pt.progressBarMax(4321)
        pt.progressBarValue(1234.0)
        self.assertEqual(pt.progressBar.value(), 1234.0)

    def test_stopExec(self):
        """test stopExec"""
        self.stop = False

        def stoppedAct(a=self):
            a.stop = True

        pt = PrintText()
        pt.stopped.connect(stoppedAct)
        self.assertTrue(pt.cancelButton.isEnabled())
        self.assertFalse(self.stop)
        pt.stopExec()
        self.assertFalse(pt.cancelButton.isEnabled())
        self.assertTrue(self.stop)

    def test_enableClose(self):
        """test enableClose"""
        pt = PrintText()
        self.assertFalse(pt.closeButton.isEnabled())
        self.assertTrue(pt.cancelButton.isEnabled())
        self.assertFalse(pt._wantToClose)
        pt.enableClose()
        self.assertTrue(pt.closeButton.isEnabled())
        self.assertTrue(pt._wantToClose)
        self.assertFalse(pt.cancelButton.isEnabled())
        pt = PrintText(noStopButton=True)
        self.assertFalse(pt.closeButton.isEnabled())
        self.assertFalse(pt._wantToClose)
        pt.enableClose()
        self.assertTrue(pt.closeButton.isEnabled())
        self.assertTrue(pt._wantToClose)


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestAdvImportDialog(GUITestCase):
    """Test AdvancedImportDialog"""

    def test_init(self):
        """Test __init__"""
        p = QWidget()
        with patch(
            "physbiblio.gui.dialogWindows.AdvancedImportDialog.initUI", autospec=True
        ) as _iu:
            aid = AdvancedImportDialog(p)
            self.assertIsInstance(aid, PBDialog)
            self.assertEqual(aid.parent(), p)
            _iu.assert_called_once_with(aid)

    def test_onCancel(self):
        """test onCancel"""
        aid = AdvancedImportDialog()
        with patch("PySide2.QtWidgets.QDialog.close", autospec=True) as _c:
            aid.onCancel()
            self.assertFalse(aid.result)
            self.assertEqual(_c.call_count, 1)

    def test_onOk(self):
        """test onOk"""
        aid = AdvancedImportDialog()
        with patch("PySide2.QtWidgets.QDialog.close", autospec=True) as _c:
            aid.onOk()
            self.assertTrue(aid.result)
            self.assertEqual(_c.call_count, 1)

    def test_initUI(self):
        """test initUI"""
        with patch(
            "physbiblio.gui.commonClasses.PBDialog.centerWindow", autospec=True
        ) as _cw:
            aid = AdvancedImportDialog()
            _cw.assert_called_once_with(aid)
        self.assertEqual(aid.windowTitle(), "Advanced import")
        self.assertIsInstance(aid.layout(), QGridLayout)
        self.assertEqual(aid.grid, aid.layout())
        self.assertEqual(aid.grid.spacing(), 1)

        self.assertIsInstance(aid.grid.itemAtPosition(0, 0).widget(), PBLabel)
        self.assertEqual(
            aid.grid.itemAtPosition(0, 0).widget().text(), "Search string: "
        )
        self.assertIsInstance(aid.grid.itemAtPosition(1, 0).widget(), PBLabel)
        self.assertEqual(
            aid.grid.itemAtPosition(1, 0).widget().text(), "Select method: "
        )

        le = aid.grid.itemAtPosition(0, 1).widget()
        self.assertIsInstance(le, QLineEdit)
        self.assertEqual(le, aid.searchStr)
        self.assertEqual(le.text(), "")
        cm = aid.grid.itemAtPosition(1, 1).widget()
        self.assertIsInstance(cm, PBComboBox)
        self.assertEqual(cm, aid.comboMethod)
        self.assertEqual(cm.count(), 5)
        self.assertEqual(cm.itemText(0), "INSPIRE-HEP")
        self.assertEqual(cm.currentText(), "INSPIRE-HEP")
        self.assertEqual(cm.itemText(1), "ADS-NASA")
        self.assertEqual(cm.itemText(2), "arXiv")
        self.assertEqual(cm.itemText(3), "DOI")
        self.assertEqual(cm.itemText(4), "ISBN")

        self.assertIsInstance(aid.grid.itemAtPosition(2, 0).widget(), QPushButton)
        self.assertEqual(aid.grid.itemAtPosition(2, 0).widget(), aid.acceptButton)
        self.assertEqual(aid.acceptButton.text(), "OK")
        with patch(
            "physbiblio.gui.dialogWindows.AdvancedImportDialog.onOk", autospec=True
        ) as _o:
            QTest.mouseClick(aid.acceptButton, Qt.LeftButton)
            _o.assert_called_once_with(aid)

        self.assertIsInstance(aid.grid.itemAtPosition(2, 1).widget(), QPushButton)
        self.assertEqual(aid.grid.itemAtPosition(2, 1).widget(), aid.cancelButton)
        self.assertEqual(aid.cancelButton.text(), "Cancel")
        self.assertTrue(aid.cancelButton.autoDefault())
        with patch(
            "physbiblio.gui.dialogWindows.AdvancedImportDialog.onCancel", autospec=True
        ) as _c:
            QTest.mouseClick(aid.cancelButton, Qt.LeftButton)
            _c.assert_called_once_with(aid)


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestAdvImportSelect(GUITestCase):
    """Test AdvancedImportSelect"""

    def test_init(self):
        """test init"""
        p = QWidget()
        with patch(
            "physbiblio.gui.dialogWindows.AdvancedImportSelect.initUI", autospec=True
        ) as _i:
            ais = AdvancedImportSelect({"abc": "def"}, p)
            _i.assert_called_once_with(ais)
        self.assertIsInstance(ais, ObjListWindow)
        self.assertEqual(ais.bibs, {"abc": "def"})
        self.assertEqual(ais.parent(), p)
        self.assertEqual(ais.checkBoxes, [])

    def test_onCancel(self):
        """test onCancel"""
        ais = AdvancedImportSelect()
        with patch("PySide2.QtWidgets.QDialog.close", autospec=True) as _c:
            ais.onCancel()
            self.assertFalse(ais.result)
            self.assertEqual(_c.call_count, 1)

    def test_onOk(self):
        """test onOk"""
        ais = AdvancedImportSelect()
        with patch("PySide2.QtWidgets.QDialog.close", autospec=True) as _c:
            ais.onOk()
            self.assertTrue(ais.result)
            self.assertEqual(_c.call_count, 1)

    def test_keyPressEvent(self):
        """test keyPressEvent"""
        ais = AdvancedImportSelect()
        ais.result = True
        with patch("PySide2.QtWidgets.QDialog.close", autospec=True) as _oc:
            QTest.keyPress(ais, "a")
            _oc.assert_not_called()
            self.assertTrue(ais.result)
            QTest.keyPress(ais, Qt.Key_Enter)
            _oc.assert_not_called()
            self.assertTrue(ais.result)
            QTest.keyPress(ais, Qt.Key_Escape)
            self.assertEqual(_oc.call_count, 1)
            self.assertFalse(ais.result)

    def test_initUI(self):
        """test initUI"""
        p = QWidget()
        ais = AdvancedImportSelect(
            {
                "abc": {
                    "exist": True,
                    "bibpars": {
                        "ID": "abc",
                        "title": "Abc",
                        "authors": "s\ng",
                        "eprint": "1807.15000",
                        "doi": "1/2/3",
                    },
                },
                "def": {
                    "exist": False,
                    "bibpars": {
                        "ID": "def",
                        "title": "De\nf",
                        "author": "SG",
                        "arxiv": "1807.16000",
                        "doi": "3/2/1",
                    },
                },
            },
            p,
        )
        self.assertEqual(ais.windowTitle(), "Advanced import - results")
        self.assertEqual(ais.layout(), ais.currLayout)
        self.assertEqual(ais.currLayout.spacing(), 1)
        self.assertIsInstance(ais.layout().itemAtPosition(0, 0).widget(), PBLabel)
        self.assertEqual(
            ais.layout().itemAtPosition(0, 0).widget().text(),
            "This is the list of elements found."
            + "\nSelect the ones that you want to import:",
        )
        self.assertIsInstance(ais.tableModel, PBImportedTableModel)
        self.assertEqual(
            ais.tableModel.header, ["ID", "title", "author", "eprint", "doi"]
        )
        self.assertEqual(ais.tableModel.bibsOrder, ["abc", "def"])
        self.assertEqual(
            ais.tableModel.dataList,
            [
                {
                    "ID": "abc",
                    "title": "Abc",
                    "authors": "s\ng",
                    "author": "s g",
                    "eprint": "1807.15000",
                    "doi": "1/2/3",
                },
                {
                    "ID": "def",
                    "title": "De f",
                    "author": "SG",
                    "eprint": "1807.16000",
                    "arxiv": "1807.16000",
                    "doi": "3/2/1",
                },
            ],
        )
        self.assertEqual(ais.tableModel.existList, [True, False])

        self.assertIsInstance(ais.layout().itemAtPosition(3, 0).widget(), QCheckBox)
        self.assertEqual(
            ais.layout().itemAtPosition(3, 0).widget().text(),
            "Ask categories at the end?",
        )
        self.assertEqual(ais.layout().itemAtPosition(3, 0).widget(), ais.askCats)
        self.assertTrue(ais.layout().itemAtPosition(3, 0).widget().isChecked())
        self.assertIsInstance(ais.layout().itemAtPosition(4, 0).widget(), QPushButton)
        self.assertEqual(ais.layout().itemAtPosition(4, 0).widget(), ais.acceptButton)
        self.assertEqual(ais.acceptButton.text(), "OK")
        self.assertIsInstance(ais.layout().itemAtPosition(5, 0).widget(), QPushButton)
        self.assertEqual(ais.layout().itemAtPosition(5, 0).widget(), ais.cancelButton)
        self.assertEqual(ais.cancelButton.text(), "Cancel")
        self.assertTrue(ais.cancelButton.autoDefault())
        with patch(
            "physbiblio.gui.dialogWindows.AdvancedImportSelect.onOk", autospec=True
        ) as _o:
            QTest.mouseClick(ais.acceptButton, Qt.LeftButton)
            _o.assert_called_once_with(ais)
        with patch(
            "physbiblio.gui.dialogWindows.AdvancedImportSelect.onCancel", autospec=True
        ) as _o:
            QTest.mouseClick(ais.cancelButton, Qt.LeftButton)
            _o.assert_called_once_with(ais)

        with patch(
            "physbiblio.gui.commonClasses.ObjListWindow.addFilterInput", autospec=True
        ) as _afi, patch(
            "physbiblio.gui.commonClasses.ObjListWindow.setProxyStuff", autospec=True
        ) as _sps, patch(
            "physbiblio.gui.commonClasses.ObjListWindow.finalizeTable", autospec=True
        ) as _ft:
            ais.initUI()
            _afi.assert_called_once_with(ais, "Filter entries", gridPos=(1, 0))
            _sps.assert_called_once_with(ais, 0, Qt.AscendingOrder)
            _ft.assert_called_once_with(ais, gridPos=(2, 0, 1, 2))

    def test_more(self):
        """test some empty functions"""
        ais = AdvancedImportSelect()
        self.assertEqual(
            None, ais.triggeredContextMenuEvent(0, 0, QEvent(QEvent.ContextMenu))
        )
        self.assertEqual(None, ais.handleItemEntered(QModelIndex()))
        self.assertEqual(None, ais.cellClick(QModelIndex()))
        self.assertEqual(None, ais.cellDoubleClick(QModelIndex()))


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestDailyArxivDialog(GUITestCase):
    """Test DailyArxivDialog"""

    def test_init(self):
        """Test __init__"""
        p = QWidget()
        with patch(
            "physbiblio.gui.dialogWindows.DailyArxivDialog.initUI", autospec=True
        ) as _iu:
            dad = DailyArxivDialog(p)
            self.assertIsInstance(dad, PBDialog)
            self.assertEqual(dad.parent(), p)
            _iu.assert_called_once_with(dad)

    def test_onCancel(self):
        """test onCancel"""
        dad = DailyArxivDialog()
        with patch("PySide2.QtWidgets.QDialog.close", autospec=True) as _c:
            dad.onCancel()
            self.assertFalse(dad.result)
            self.assertEqual(_c.call_count, 1)

    def test_onOk(self):
        """test onOk"""
        dad = DailyArxivDialog()
        with patch("PySide2.QtWidgets.QDialog.close", autospec=True) as _c:
            dad.onOk()
            self.assertTrue(dad.result)
            self.assertEqual(_c.call_count, 1)

    def test_updateCat(self):
        """test updateCat"""
        dad = DailyArxivDialog()
        self.assertEqual(dad.comboSub.count(), 1)
        self.assertEqual(dad.comboSub.itemText(0), "")
        for c in physBiblioWeb.webSearch["arxiv"].categories.keys():
            dad.updateCat(c)
            self.assertEqual(
                dad.comboSub.count(),
                1 + len(physBiblioWeb.webSearch["arxiv"].categories[c]),
            )
            self.assertEqual(dad.comboSub.itemText(0), "--")
            for ix, sc in enumerate(physBiblioWeb.webSearch["arxiv"].categories[c]):
                self.assertEqual(dad.comboSub.itemText(ix + 1), sc)

    def test_initUI(self):
        """test initUI"""
        dad = DailyArxivDialog()
        self.assertEqual(dad.windowTitle(), "Browse arxiv daily")
        self.assertIsInstance(dad.layout(), QGridLayout)
        self.assertEqual(dad.grid, dad.layout())
        self.assertEqual(dad.grid.spacing(), 1)

        self.assertIsInstance(dad.grid.itemAtPosition(0, 0).widget(), PBLabel)
        self.assertEqual(
            dad.grid.itemAtPosition(0, 0).widget().text(), "Select category: "
        )
        self.assertIsInstance(dad.grid.itemAtPosition(1, 0).widget(), PBLabel)
        self.assertEqual(dad.grid.itemAtPosition(1, 0).widget().text(), "Subcategory: ")

        cc = dad.grid.itemAtPosition(0, 1).widget()
        self.assertIsInstance(cc, PBComboBox)
        self.assertEqual(cc, dad.comboCat)
        self.assertEqual(
            cc.count(), len(physBiblioWeb.webSearch["arxiv"].categories) + 1
        )
        self.assertEqual(cc.itemText(0), "")
        for ix, c in enumerate(
            sorted(physBiblioWeb.webSearch["arxiv"].categories.keys())
        ):
            self.assertEqual(cc.itemText(ix + 1), c)
        with patch(
            "physbiblio.gui.dialogWindows.DailyArxivDialog.updateCat", autospec=True
        ) as _cic:
            dad.comboCat.setCurrentText("hep-ph")
            _cic.assert_called_once_with(dad, "hep-ph")
        cs = dad.grid.itemAtPosition(1, 1).widget()
        self.assertIsInstance(cs, PBComboBox)
        self.assertEqual(cs, dad.comboSub)
        self.assertEqual(cs.count(), 1)
        self.assertEqual(cs.itemText(0), "")

        self.assertIsInstance(dad.grid.itemAtPosition(2, 0).widget(), QPushButton)
        self.assertEqual(dad.grid.itemAtPosition(2, 0).widget(), dad.acceptButton)
        self.assertEqual(dad.acceptButton.text(), "OK")
        with patch(
            "physbiblio.gui.dialogWindows.DailyArxivDialog.onOk", autospec=True
        ) as _o:
            QTest.mouseClick(dad.acceptButton, Qt.LeftButton)
            _o.assert_called_once_with(dad)

        self.assertIsInstance(dad.grid.itemAtPosition(2, 1).widget(), QPushButton)
        self.assertEqual(dad.grid.itemAtPosition(2, 1).widget(), dad.cancelButton)
        self.assertEqual(dad.cancelButton.text(), "Cancel")
        self.assertTrue(dad.cancelButton.autoDefault())
        with patch("physbiblio.gui.dialogWindows.DailyArxivDialog.onCancel") as _c:
            QTest.mouseClick(dad.cancelButton, Qt.LeftButton)
            _c.assert_called_once_with()

        with patch(
            "PySide2.QtWidgets.QDialog.frameGeometry",
            autospec=True,
            return_value=QRect(),
        ) as _fg, patch(
            "PySide2.QtCore.QRect.center", autospec=True, return_value=QPoint()
        ) as _ce, patch(
            "PySide2.QtCore.QRect.moveCenter", autospec=True
        ) as _mc, patch(
            "PySide2.QtCore.QRect.topLeft", autospec=True, return_value=QPoint()
        ) as _tl, patch(
            "PySide2.QtWidgets.QDialog.move", autospec=True
        ) as _mo:
            dad.initUI()
            self.assertEqual(_fg.call_count, 1)
            self.assertEqual(_ce.call_count, 1)
            self.assertEqual(_mc.call_count, 1)
            self.assertEqual(_tl.call_count, 1)
            self.assertEqual(_mo.call_count, 1)


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestDailyArxivSelect(GUITestCase):
    """Test DailyArxivSelect"""

    def test_init(self):
        """test init"""
        p = QWidget()
        das = DailyArxivSelect({}, p)
        self.assertIsInstance(das, AdvancedImportSelect)

    def test_initUI(self):
        """test initUI"""
        p = QWidget()
        with patch(
            "PySide2.QtWidgets.QTableView.sortByColumn", autospec=True
        ) as _s, patch(
            "physbiblio.gui.commonClasses.ObjListWindow.finalizeTable", autospec=True
        ) as _f:
            das = DailyArxivSelect({}, p)
            _s.assert_called_once_with(0, Qt.AscendingOrder)
            _f.assert_called_once_with(das, gridPos=(2, 0, 1, 2))
        das = DailyArxivSelect({}, p)
        self.assertEqual(das.windowTitle(), "ArXiv daily listing - results")
        self.assertEqual(das.layout().spacing(), 1)
        self.assertIsInstance(das.layout().itemAt(0).widget(), PBLabel)
        self.assertEqual(
            das.layout().itemAt(0).widget().text(),
            "This is the list of elements found.\n"
            + "Select the ones that you want to import:",
        )
        self.assertIsInstance(das.tableModel, PBImportedTableModel)
        self.assertEqual(
            das.tableModel.header,
            ["eprint", "type", "title", "author", "primaryclass", "abstract"],
        )
        self.assertEqual(das.tableModel.idName, "eprint")
        self.assertIsInstance(das.layout().itemAt(1).widget(), QLineEdit)
        self.assertEqual(das.layout().itemAt(1).widget(), das.filterInput)
        self.assertIsInstance(das.layout().itemAt(2).widget(), PBTableView)
        self.assertEqual(das.layout().itemAt(2).widget(), das.tableview)
        self.assertIsInstance(das.askCats, QCheckBox)
        self.assertEqual(das.askCats.text(), "Ask categories at the end?")
        self.assertTrue(das.askCats.isChecked())
        self.assertEqual(das.layout().itemAt(3).widget(), das.askCats)
        self.assertIsInstance(das.layout().itemAt(4).widget(), QPushButton)
        self.assertEqual(das.layout().itemAt(4).widget(), das.acceptButton)
        self.assertEqual(das.acceptButton.text(), "OK")
        self.assertIsInstance(das.layout().itemAt(5).widget(), QPushButton)
        self.assertEqual(das.layout().itemAt(5).widget(), das.cancelButton)
        self.assertEqual(das.cancelButton.text(), "Cancel")
        self.assertTrue(das.cancelButton.autoDefault())
        with patch(
            "physbiblio.gui.dialogWindows.AdvancedImportSelect.onOk", autospec=True
        ) as _o:
            QTest.mouseClick(das.acceptButton, Qt.LeftButton)
            _o.assert_called_once_with(das)
        with patch(
            "physbiblio.gui.dialogWindows.AdvancedImportSelect.onCancel", autospec=True
        ) as _o:
            QTest.mouseClick(das.cancelButton, Qt.LeftButton)
            _o.assert_called_once_with(das)
        self.assertIsInstance(das.layout().itemAt(6).widget(), QTextEdit)
        self.assertEqual(das.layout().itemAt(6).widget(), das.abstractArea)
        self.assertEqual(das.abstractArea.toPlainText(), "Abstract")

    def test_cellClick(self):
        """test cellClick"""
        p = QWidget()
        bibs = {
            "1808.00000": {
                "bibpars": {
                    "eprint": "1808.00000",
                    "type": "art",
                    "title": "empty",
                    "author": "me",
                    "primaryclass": "physics",
                    "abstract": "no text",
                },
                "exist": False,
            },
            "1507.08204": {
                "bibpars": {
                    "eprint": "1507.08204",
                    "type": "art",
                    "title": "Light sterile neutrinos",
                    "author": "SG",
                    "primaryclass": "hep-ph",
                    "abstract": "some text",
                },
                "exist": False,
            },
        }
        das = DailyArxivSelect(bibs, p)
        with patch(
            "PySide2.QtCore.QModelIndex.isValid", return_value=False, autospec=True
        ) as _iv:
            self.assertEqual(das.cellClick(QModelIndex()), None)
        self.assertFalse(hasattr(das, "abstractFormulas"))
        # the first row will contain the 1507.08204 entry,
        # as the order is Qt.AscendingOrder on the first column (eprint)
        ix = das.tableview.model().index(1, 0)
        with patch("logging.Logger.debug") as _d:
            das.cellClick(ix)
            _d.assert_called_once_with(
                "self.abstractFormulas not present "
                + "in DailyArxivSelect. Eprint: 1808.00000"
            )
        with patch(
            "PySide2.QtCore.QSortFilterProxyModel.sibling",
            return_value=None,
            autospec=True,
        ) as _s, patch("logging.Logger.debug") as _d:
            self.assertEqual(das.cellClick(ix), None)
            _d.assert_called_once_with("Data not valid", exc_info=True)
            _s.assert_called_once_with(1, 0, ix)
        ix = das.tableview.model().index(0, 0)
        das.abstractFormulas = Fake_abstractFormulas()
        with patch(
            "physbiblio.gui.bibWindows.AbstractFormulas.doText", autospec=True
        ) as _d:
            das.cellClick(ix)
            self.assertIsInstance(das.abstractFormulas.el, AbstractFormulas)
            self.assertEqual(das.abstractFormulas.par, p)
            self.assertEqual(das.abstractFormulas.abstract, "some text")
            self.assertEqual(das.abstractFormulas.ce, das.abstractArea)
            self.assertEqual(das.abstractFormulas.sm, False)
            _d.assert_called_once_with(das.abstractFormulas.el)


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestExportForTexDialog(GUITestCase):
    """Test ExportForTexDialog"""

    def test_init(self):
        """test init"""
        with patch(
            "physbiblio.gui.dialogWindows.ExportForTexDialog.initUI", autospec=True
        ) as _iu:
            eft = ExportForTexDialog()
            _iu.assert_called_once_with(eft)
        self.assertIsInstance(eft, PBDialog)
        self.assertEqual(eft.numTexFields, 1)
        self.assertEqual(eft.bibName, "")
        self.assertEqual(eft.texFileNames, [""])
        self.assertEqual(eft.texNames, [])
        self.assertFalse(eft.update)
        self.assertFalse(eft.remove)
        self.assertFalse(eft.result)
        self.assertIsInstance(eft.grid, QGridLayout)
        self.assertEqual(eft.layout(), eft.grid)

    def test_readForm(self):
        """test readForm"""
        eft = ExportForTexDialog()
        eft.onAddTex()
        eft.bibButton.setText("file.bib")
        eft.updateCheck.setChecked(True)
        eft.removeCheck.setChecked(False)
        eft.texButtons[0].setText("%s" % ["a.tex", "b.tex", "", "Select file"])
        eft.texButtons[1].setText("c.tex")
        eft.readForm()
        self.assertEqual(eft.update, True)
        self.assertEqual(eft.remove, False)
        self.assertEqual(eft.bibName, "file.bib")
        self.assertEqual(
            eft.texFileNames, [["a.tex", "b.tex", "", "Select file"], "c.tex"]
        )
        self.assertEqual(eft.texNames, ["a.tex", "b.tex", "c.tex"])
        eft.bibButton.setText("Select file")
        eft.readForm()
        self.assertEqual(eft.bibName, "")

    def test_onAddTex(self):
        """test onAddTex"""
        eft = ExportForTexDialog()
        eft.numTexFields = 3
        with patch(
            "physbiblio.gui.dialogWindows.ExportForTexDialog.cleanLayout", autospec=True
        ) as _cl, patch(
            "physbiblio.gui.dialogWindows.ExportForTexDialog.initUI", autospec=True
        ) as _iu, patch(
            "physbiblio.gui.dialogWindows.ExportForTexDialog.readForm", autospec=True
        ) as _rf:
            eft.onAddTex()
            self.assertEqual(eft.numTexFields, 4)
            self.assertEqual(eft.texFileNames, ["", "", "", ""])
            _cl.assert_called_once_with(eft)
            _iu.assert_called_once_with(eft)
            _rf.assert_called_once_with(eft)

    def test_onCancel(self):
        """test onCancel"""
        eow = ExportForTexDialog()
        with patch("PySide2.QtWidgets.QDialog.close", autospec=True) as _c:
            eow.onCancel()
            self.assertFalse(eow.result)
            self.assertEqual(_c.call_count, 1)

    def test_onOk(self):
        """test onOk"""
        eow = ExportForTexDialog()
        with patch("PySide2.QtWidgets.QDialog.close", autospec=True) as _c, patch(
            "physbiblio.gui.dialogWindows.ExportForTexDialog.readForm", autospec=True
        ) as _rf:
            eow.onOk()
            self.assertTrue(eow.result)
            _rf.assert_called_once_with(eow)
            self.assertEqual(_c.call_count, 1)

    def test_onAskBib(self):
        """test onAskBib"""
        eft = ExportForTexDialog()
        with patch(
            "physbiblio.gui.dialogWindows.askSaveFileName",
            return_value="",
            autospec=True,
        ) as _af:
            eft.onAskBib()
        self.assertEqual(eft.bibButton.text(), "Select file")
        with patch(
            "physbiblio.gui.dialogWindows.askSaveFileName",
            return_value="f.bib",
            autospec=True,
        ) as _af:
            eft.onAskBib()
        self.assertEqual(eft.bibButton.text(), "f.bib")

    def test_onAskTex(self):
        """test onAskTex"""
        eft = ExportForTexDialog()
        eft.onAddTex()
        with patch(
            "physbiblio.gui.dialogWindows.askFileNames", return_value="", autospec=True
        ) as _af:
            eft.onAskTex(0)
        self.assertEqual(eft.texButtons[0].text(), "Select file")
        with patch(
            "physbiblio.gui.dialogWindows.askFileNames", return_value=[], autospec=True
        ) as _af:
            eft.onAskTex(0)
        self.assertEqual(eft.texButtons[0].text(), "Select file")
        with patch(
            "physbiblio.gui.dialogWindows.askFileNames",
            return_value="a.tex",
            autospec=True,
        ) as _af:
            eft.onAskTex(0)
        self.assertEqual(eft.texButtons[0].text(), "a.tex")
        with patch(
            "physbiblio.gui.dialogWindows.askFileNames",
            return_value=["b.tex"],
            autospec=True,
        ) as _af:
            eft.onAskTex(1)
        self.assertEqual(eft.texButtons[1].text(), "['b.tex']")

    def test_initUI(self):
        """test initUI"""
        eft = ExportForTexDialog()
        self.assertEqual(eft.numTexFields, 1)
        self.assertEqual(eft.bibName, "")
        self.assertEqual(eft.texFileNames, [""])
        self.assertEqual(eft.windowTitle(), "Export for tex file")
        self.assertIsInstance(eft.grid.itemAtPosition(0, 0).widget(), PBLabelRight)
        self.assertEqual(
            eft.grid.itemAtPosition(0, 0).widget().text(), "Bib file name:"
        )
        self.assertIsInstance(eft.grid.itemAtPosition(0, 2).widget(), QPushButton)
        self.assertEqual(eft.grid.itemAtPosition(0, 2).widget(), eft.bibButton)
        self.assertEqual(eft.bibButton.text(), "Select file")
        with patch(
            "physbiblio.gui.dialogWindows.ExportForTexDialog.onAskBib", autospec=True
        ) as _f:
            QTest.mouseClick(eft.bibButton, Qt.LeftButton)
            _f.assert_called_once_with(eft)

        self.assertIsInstance(eft.grid.itemAtPosition(1, 0).widget(), PBLabelRight)
        self.assertEqual(
            eft.grid.itemAtPosition(1, 0).widget().text(), "Tex file name(s):"
        )
        self.assertIsInstance(eft.texButtons, list)
        self.assertEqual(len(eft.texButtons), eft.numTexFields)
        self.assertIsInstance(eft.grid.itemAtPosition(1, 2).widget(), QPushButton)
        self.assertEqual(eft.grid.itemAtPosition(1, 2).widget(), eft.texButtons[0])
        self.assertEqual(eft.texButtons[0].text(), "Select file")
        with patch(
            "physbiblio.gui.dialogWindows.ExportForTexDialog.onAskTex", autospec=True
        ) as _f:
            QTest.mouseClick(eft.texButtons[0], Qt.LeftButton)
            _f.assert_called_once_with(eft, 0)

        self.assertIsInstance(eft.grid.itemAtPosition(2, 2).widget(), QPushButton)
        self.assertEqual(eft.grid.itemAtPosition(2, 2).widget(), eft.addTexButton)
        self.assertEqual(eft.addTexButton.text(), "Add more tex files")
        with patch(
            "physbiblio.gui.dialogWindows.ExportForTexDialog.onAddTex", autospec=True
        ) as _f:
            QTest.mouseClick(eft.addTexButton, Qt.LeftButton)
            _f.assert_called_once_with(eft)

        self.assertIsInstance(eft.grid.itemAtPosition(3, 0).widget(), QCheckBox)
        self.assertEqual(eft.grid.itemAtPosition(3, 0).widget(), eft.removeCheck)
        self.assertEqual(eft.removeCheck.text(), "Remove unused bibtexs?")
        self.assertFalse(eft.removeCheck.isChecked())
        self.assertIsInstance(eft.grid.itemAtPosition(3, 2).widget(), QCheckBox)
        self.assertEqual(eft.grid.itemAtPosition(3, 2).widget(), eft.updateCheck)
        self.assertEqual(eft.updateCheck.text(), "Update existing bibtexs?")
        self.assertFalse(eft.updateCheck.isChecked())

        self.assertIsInstance(eft.grid.itemAtPosition(4, 1).widget(), QPushButton)
        self.assertEqual(eft.grid.itemAtPosition(4, 1).widget(), eft.acceptButton)
        self.assertEqual(eft.acceptButton.text(), "OK")
        with patch(
            "physbiblio.gui.dialogWindows.ExportForTexDialog.onOk", autospec=True
        ) as _f:
            QTest.mouseClick(eft.acceptButton, Qt.LeftButton)
            _f.assert_called_once_with(eft)

        self.assertIsInstance(eft.grid.itemAtPosition(4, 2).widget(), QPushButton)
        self.assertEqual(eft.grid.itemAtPosition(4, 2).widget(), eft.cancelButton)
        self.assertEqual(eft.cancelButton.text(), "Cancel")
        with patch(
            "physbiblio.gui.dialogWindows.ExportForTexDialog.onCancel", autospec=True
        ) as _f:
            QTest.mouseClick(eft.cancelButton, Qt.LeftButton)
            _f.assert_called_once_with(eft)

        with patch(
            "physbiblio.gui.dialogWindows.ExportForTexDialog.initUI", autospec=True
        ) as _iu:
            eft = ExportForTexDialog()
        eft.numTexFields = 2
        eft.bibName = "file.bib"
        eft.texFileNames = ["a.tex", ""]
        eft.update = True
        eft.remove = True
        with patch(
            "physbiblio.gui.dialogWindows.ExportForTexDialog.centerWindow",
            autospec=True,
        ) as _cw, patch(
            "physbiblio.gui.dialogWindows.ExportForTexDialog.setGeometry", autospec=True
        ) as _sg:
            eft.initUI()
            _cw.assert_called_once_with(eft)
            _sg.assert_called_once_with(100, 100, 400, 150)
        self.assertEqual(eft.bibButton.text(), "file.bib")

        self.assertIsInstance(eft.grid.itemAtPosition(1, 0).widget(), PBLabelRight)
        self.assertEqual(
            eft.grid.itemAtPosition(1, 0).widget().text(), "Tex file name(s):"
        )
        self.assertIsInstance(eft.texButtons, list)
        self.assertEqual(len(eft.texButtons), eft.numTexFields)
        self.assertIsInstance(eft.grid.itemAtPosition(1, 2).widget(), QPushButton)
        self.assertEqual(eft.grid.itemAtPosition(1, 2).widget(), eft.texButtons[0])
        self.assertEqual(eft.texButtons[0].text(), "a.tex")
        with patch(
            "physbiblio.gui.dialogWindows.ExportForTexDialog.onAskTex", autospec=True
        ) as _f:
            QTest.mouseClick(eft.texButtons[0], Qt.LeftButton)
            _f.assert_called_once_with(eft, 0)
        self.assertIsInstance(eft.grid.itemAtPosition(2, 0).widget(), PBLabelRight)
        self.assertEqual(
            eft.grid.itemAtPosition(2, 0).widget().text(), "Tex file name(s):"
        )
        self.assertIsInstance(eft.grid.itemAtPosition(2, 2).widget(), QPushButton)
        self.assertEqual(eft.grid.itemAtPosition(2, 2).widget(), eft.texButtons[1])
        self.assertEqual(eft.texButtons[1].text(), "Select file")
        with patch(
            "physbiblio.gui.dialogWindows.ExportForTexDialog.onAskTex", autospec=True
        ) as _f:
            QTest.mouseClick(eft.texButtons[1], Qt.LeftButton)
            _f.assert_called_once_with(eft, 1)
        self.assertEqual(eft.grid.itemAtPosition(3, 2).widget(), eft.addTexButton)
        self.assertEqual(eft.grid.itemAtPosition(4, 0).widget(), eft.removeCheck)
        self.assertTrue(eft.removeCheck.isChecked())
        self.assertEqual(eft.grid.itemAtPosition(4, 2).widget(), eft.updateCheck)
        self.assertTrue(eft.updateCheck.isChecked())
        self.assertEqual(eft.grid.itemAtPosition(5, 1).widget(), eft.acceptButton)
        self.assertEqual(eft.grid.itemAtPosition(5, 2).widget(), eft.cancelButton)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""Test file for the physbiblio.gui.commonClasses module.

This file is part of the physbiblio package.
"""
import sys
import traceback
import os
import time
from PySide2.QtCore import (
    Qt,
    QByteArray,
    QItemSelectionModel,
    QMimeData,
    QModelIndex,
    QPoint,
    QPointF,
    QRect,
)
from PySide2.QtGui import QContextMenuEvent, QDropEvent
from PySide2.QtTest import QTest
from PySide2.QtWidgets import QAction, QDesktopWidget, QInputDialog, QLineEdit, QWidget

if sys.version_info[0] < 3:
    import unittest2 as unittest
    from mock import patch, call
    from Queue import Queue
else:
    import unittest
    from unittest.mock import patch, call
    from queue import Queue

try:
    from physbiblio.setuptests import *
    from physbiblio.database import pBDB
    from physbiblio.gui.setuptests import *
    from physbiblio.gui.commonClasses import *
except ImportError:
    print("Could not find physbiblio and its modules!")
    raise
except Exception:
    print(traceback.format_exc())


def fakeExec_writeStream_mys(x, text):
    """simulate WriteStream.run and set running=False"""
    x.running = False
    x.text = text


def fakeExec_writeStream_fin(x):
    """probe that finished has been emitted"""
    x.running = False
    x.fin = True


def fakeExec_dataChanged(x, y):
    """probe that dataChanged has been emitted"""
    assert x == y


class EmptyTableModel(QAbstractTableModel):
    """Used to do tests when a table model is needed"""

    def __init__(self, *args):
        QAbstractTableModel.__init__(self, *args)
        self.header = []

    def rowCount(self, a=None):
        return 1

    def columnCount(self, a=None):
        return 1

    def data(self, index, role):
        return None


class EmptyTreeModel(TreeModel):
    """Used to do tests when a tree model is needed"""

    def __init__(self, elements, *args):
        self.rootElements = elements
        TreeModel.__init__(self, *args)

    def _getRootNodes(self):
        return [
            NamedNode(elem, None, index) for index, elem in enumerate(self.rootElements)
        ]

    def columnCount(self, a=None):
        return 1

    def data(self, index, role):
        if not index.isValid():
            return None
        if role == Qt.DisplayRole and index.column() == 0:
            return TreeNode.cast(index).element.text
        return None


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestPBDialog(GUITestCase):
    """Test the PBDialog class"""

    def test_centerWindow(self):
        """test centerWindow"""
        p = PBDialog()
        with patch(
            "PySide2.QtWidgets.QDialog.frameGeometry",
            autospec=True,
            return_value=QRect(),
        ) as _fg, patch(
            "PySide2.QtCore.QRect.center", autospec=True, return_value=QPoint()
        ) as _ce, patch(
            "PySide2.QtCore.QRect.moveCenter", autospec=True
        ) as _mc, patch(
            "PySide2.QtCore.QRect.topLeft", autospec=True, return_value=QPoint()
        ) as _tl, patch(
            "PySide2.QtWidgets.QDialog.move", autospec=True
        ) as _mo:
            p.centerWindow()
            self.assertEqual(_fg.call_count, 1)
            self.assertEqual(_ce.call_count, 1)
            self.assertEqual(_mc.call_count, 1)
            self.assertEqual(_tl.call_count, 1)
            self.assertEqual(_mo.call_count, 1)

    def test_cleanLayout(self):
        """Test cleanLayout"""
        p = PBDialog()
        p.currLayout = QGridLayout()
        p.setLayout(p.currLayout)
        p.layout().addWidget(QLabel("empty"))
        p.layout().addWidget(QLabel("empty1"))
        self.assertEqual(p.layout().count(), 2)
        p.cleanLayout()
        self.assertEqual(p.layout().count(), 0)


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestLabels(GUITestCase):
    """Test the PBLabelRight and PBLabelCenter classes"""

    def test_myLabel(self):
        """Test PBLabel"""
        l = PBLabel("label")
        self.assertIsInstance(l, QLabel)
        self.assertEqual(l.text(), "label")
        self.assertEqual(
            l.textInteractionFlags(),
            Qt.LinksAccessibleByMouse | Qt.TextSelectableByMouse,
        )
        self.assertEqual(l.textFormat(), Qt.RichText)
        self.assertEqual(l.openExternalLinks(), True)

    def test_myLabelRight(self):
        """Test PBLabelRight"""
        l = PBLabelRight("label")
        self.assertIsInstance(l, QLabel)
        self.assertEqual(l.text(), "label")
        self.assertEqual(l.alignment(), Qt.AlignRight | Qt.AlignVCenter)

    def test_myLabelCenter(self):
        """Test PBLabelCenter"""
        l = PBLabelCenter("label")
        self.assertIsInstance(l, QLabel)
        self.assertEqual(l.text(), "label")
        self.assertEqual(l.alignment(), Qt.AlignCenter | Qt.AlignVCenter)


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestPBComboBox(GUITestCase):
    """Test the PBComboBox class"""

    def test_init(self):
        """test the constructor"""
        ew = QWidget()
        mb = PBComboBox(ew, [])
        self.assertIsInstance(mb, QComboBox)
        self.assertEqual(mb.parentWidget(), ew)
        self.assertEqual(mb.itemText(0), "")
        mb = PBComboBox(ew, ["1", "2"])
        self.assertEqual(mb.itemText(0), "1")
        self.assertEqual(mb.itemText(1), "2")
        self.assertEqual(mb.itemText(2), "")
        self.assertEqual(mb.currentText(), "1")
        mb = PBComboBox(ew, ["1", "2"], "0")
        self.assertEqual(mb.currentText(), "1")
        mb = PBComboBox(ew, ["1", "2"], "2")
        self.assertEqual(mb.currentText(), "2")
        mb = PBComboBox(ew, [1, 2])
        self.assertEqual(mb.itemText(0), "1")
        self.assertEqual(mb.itemText(1), "2")
        self.assertEqual(mb.itemText(2), "")


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestPBAndOrCombo(GUITestCase):
    """Test the PBAndOrCombo class"""

    def test_init(self):
        """test the constructor"""
        mb = PBAndOrCombo(None)
        self.assertIsInstance(mb, PBComboBox)
        self.assertIsInstance(mb, QComboBox)
        self.assertEqual(mb.parentWidget(), None)
        self.assertEqual(mb.itemText(0), "AND")
        self.assertEqual(mb.itemText(1), "OR")
        self.assertEqual(mb.itemText(2), "")
        self.assertEqual(mb.currentText(), "AND")
        mb = PBAndOrCombo(None, "or")
        self.assertEqual(mb.currentText(), "AND")
        ew = QWidget()
        mb = PBAndOrCombo(ew, "OR")
        self.assertEqual(mb.parentWidget(), ew)
        self.assertEqual(mb.currentText(), "OR")


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestPBTrueFalseCombo(GUITestCase):
    """Test the PBTrueFalseCombo class"""

    def test_init(self):
        """test the constructor"""
        mb = PBTrueFalseCombo(None)
        self.assertIsInstance(mb, PBComboBox)
        self.assertIsInstance(mb, QComboBox)
        self.assertEqual(mb.parentWidget(), None)
        self.assertEqual(mb.itemText(0), "True")
        self.assertEqual(mb.itemText(1), "False")
        self.assertEqual(mb.itemText(2), "")
        self.assertEqual(mb.currentText(), "True")
        mb = PBTrueFalseCombo(None, "abc")
        self.assertEqual(mb.currentText(), "True")
        ew = QWidget()
        mb = PBTrueFalseCombo(ew, "False")
        self.assertEqual(mb.parentWidget(), ew)
        self.assertEqual(mb.currentText(), "False")


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestObjListWindow(GUITestCase):
    """Test the ObjListWindow class"""

    def test_init(self):
        """test the __init__ function"""
        olw = ObjListWindow()
        self.assertIsInstance(olw, QDialog)
        self.assertEqual(olw.tableWidth, None)
        self.assertEqual(olw.proxyModel, None)
        self.assertFalse(olw.gridLayout)
        self.assertIsInstance(olw.currLayout, QVBoxLayout)
        self.assertEqual(olw.layout(), olw.currLayout)

        olw = ObjListWindow(gridLayout=True)
        self.assertTrue(olw.gridLayout)
        self.assertIsInstance(olw.currLayout, QGridLayout)
        self.assertEqual(olw.layout(), olw.currLayout)

    def test_NI(self):
        """Test the non implemented functions (must be subclassed!)"""
        olw = ObjListWindow()
        ix = QModelIndex()
        self.assertRaises(NotImplementedError, lambda: olw.createTable())
        self.assertRaises(NotImplementedError, lambda: olw.cellClick(ix))
        self.assertRaises(NotImplementedError, lambda: olw.cellDoubleClick(ix))
        self.assertRaises(NotImplementedError, lambda: olw.handleItemEntered(ix))
        self.assertRaises(
            NotImplementedError, lambda: olw.triggeredContextMenuEvent(0, 0, ix)
        )

    def test_changeFilter(self):
        """test changeFilter"""
        olw = ObjListWindow()
        olw.tableModel = EmptyTableModel()
        olw.setProxyStuff(1, Qt.AscendingOrder)
        olw.changeFilter("abc")
        self.assertEqual(olw.proxyModel.filterRegExp().pattern(), "abc")
        olw.changeFilter(123)
        self.assertEqual(olw.proxyModel.filterRegExp().pattern(), "123")

    def test_addFilterInput(self):
        """test addFilterInput"""
        olw = ObjListWindow()
        olw.addFilterInput("plch")
        self.assertIsInstance(olw.filterInput, QLineEdit)
        self.assertEqual(olw.filterInput.placeholderText(), "plch")
        with patch(
            "physbiblio.gui.commonClasses.ObjListWindow.changeFilter", autospec=True
        ) as _cf:
            olw.filterInput.textChanged.emit("sss")
            _cf.assert_called_once_with(olw, "sss")

        olw = ObjListWindow(gridLayout=True)
        olw.addFilterInput("plch", gridPos=(4, 1))
        self.assertEqual(olw.layout().itemAtPosition(4, 1).widget(), olw.filterInput)

    def test_setProxyStuff(self):
        """test setProxyStuff"""
        olw = ObjListWindow()
        olw.tableModel = EmptyTableModel()
        with patch(
            "PySide2.QtWidgets.QTableView.sortByColumn", autospec=True
        ) as _st, patch(
            "PySide2.QtCore.QSortFilterProxyModel.sort", autospec=True
        ) as _sf:
            olw.setProxyStuff(1, Qt.AscendingOrder)
            _st.assert_called_once_with(1, Qt.AscendingOrder)
            _sf.assert_called_once_with(1, Qt.AscendingOrder)
        self.assertIsInstance(olw.proxyModel, QSortFilterProxyModel)
        self.assertEqual(olw.proxyModel.filterCaseSensitivity(), Qt.CaseInsensitive)
        self.assertEqual(olw.proxyModel.sortCaseSensitivity(), Qt.CaseInsensitive)
        self.assertEqual(olw.proxyModel.filterKeyColumn(), -1)

        self.assertIsInstance(olw.tableview, PBTableView)
        self.assertEqual(olw.tableview.model(), olw.proxyModel)
        self.assertTrue(olw.tableview.isSortingEnabled())
        self.assertTrue(olw.tableview.hasMouseTracking())
        self.assertTrue(olw.tableview.selectionBehavior(), QAbstractItemView.SelectRows)
        self.assertEqual(olw.layout().itemAt(0).widget(), olw.tableview)

    def test_finalizeTable(self):
        """Test finalizeTable"""
        olw = ObjListWindow()
        olw.tableModel = EmptyTableModel()
        with patch("PySide2.QtCore.QSortFilterProxyModel.sort", autospec=True) as _s:
            olw.setProxyStuff(1, Qt.AscendingOrder)
        with patch(
            "PySide2.QtWidgets.QTableView.resizeColumnsToContents", autospec=True
        ) as _rc, patch(
            "PySide2.QtWidgets.QTableView.resizeRowsToContents", autospec=True
        ) as _rr:
            olw.finalizeTable()
            _rc.assert_has_calls([call(), call()])
            self.assertEqual(_rr.call_count, 1)
        self.assertIsInstance(olw.tableview, PBTableView)
        maxw = QDesktopWidget().availableGeometry().width()
        self.assertEqual(
            olw.maximumHeight(), QDesktopWidget().availableGeometry().height()
        )
        self.assertEqual(olw.maximumWidth(), maxw)
        hwidth = olw.tableview.horizontalHeader().length()
        swidth = olw.tableview.style().pixelMetric(QStyle.PM_ScrollBarExtent)
        fwidth = olw.tableview.frameWidth() * 2

        if hwidth > maxw - (swidth + fwidth):
            tW = maxw - (swidth + fwidth)
        else:
            tW = hwidth + swidth + fwidth
        self.assertEqual(olw.tableview.width(), tW)
        self.assertEqual(olw.minimumHeight(), 600)
        ix = QModelIndex()
        with patch(
            "physbiblio.gui.commonClasses.ObjListWindow.handleItemEntered",
            autospec=True,
        ) as _f:
            olw.tableview.entered.emit(ix)
            _f.assert_called_once_with(olw, ix)
        with patch(
            "physbiblio.gui.commonClasses.ObjListWindow.cellClick", autospec=True
        ) as _f:
            olw.tableview.clicked.emit(ix)
            _f.assert_called_once_with(olw, ix)
        with patch(
            "physbiblio.gui.commonClasses.ObjListWindow.cellDoubleClick", autospec=True
        ) as _f:
            olw.tableview.doubleClicked.emit(ix)
            _f.assert_called_once_with(olw, ix)
        self.assertEqual(olw.layout().itemAt(0).widget(), olw.tableview)

        olw = ObjListWindow(gridLayout=True)
        olw.tableModel = EmptyTableModel()
        with patch("PySide2.QtCore.QSortFilterProxyModel.sort", autospec=True) as _s:
            olw.setProxyStuff(1, Qt.AscendingOrder)
        olw.finalizeTable(gridPos=(4, 1))
        self.assertEqual(olw.layout().itemAtPosition(4, 1).widget(), olw.tableview)

    def test_recreateTable(self):
        """Test recreateTable"""
        olw = ObjListWindow()
        with patch(
            "physbiblio.gui.commonClasses.ObjListWindow.cleanLayout", autospec=True
        ) as _cl, patch(
            "physbiblio.gui.commonClasses.ObjListWindow.createTable", autospec=True
        ) as _ct:
            olw.recreateTable()
            self.assertEqual(_cl.call_count, 1)
            self.assertEqual(_ct.call_count, 1)


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestEditObjectWindow(GUITestCase):
    """Test the EditObjectWindow class"""

    def test_init(self):
        """test constructor"""
        ew = QWidget()
        with patch(
            "physbiblio.gui.commonClasses.EditObjectWindow.initUI", autospec=True
        ) as _iu:
            eow = EditObjectWindow(ew)
            self.assertEqual(eow.parent(), ew)
            self.assertEqual(eow.textValues, {})
            _iu.assert_called_once_with(eow)

    def test_keyPressEvent(self):
        """test keyPressEvent"""
        eow = EditObjectWindow()
        w = QLineEdit()
        eow.currGrid.addWidget(w, 0, 0)
        with patch(
            "physbiblio.gui.commonClasses.EditObjectWindow.onCancel", autospec=True
        ) as _oc:
            QTest.keyPress(w, "a")
            self.assertEqual(_oc.call_count, 0)
            QTest.keyPress(w, Qt.Key_Enter)
            self.assertEqual(_oc.call_count, 0)
            QTest.keyPress(w, Qt.Key_Escape)
            self.assertEqual(_oc.call_count, 1)

    def test_onCancel(self):
        """test onCancel"""
        eow = EditObjectWindow()
        with patch("PySide2.QtWidgets.QDialog.close", autospec=True) as _c:
            eow.onCancel()
            self.assertFalse(eow.result)
            self.assertEqual(_c.call_count, 1)

    def test_onOk(self):
        """test onOk"""
        eow = EditObjectWindow()
        with patch("PySide2.QtWidgets.QDialog.close", autospec=True) as _c:
            eow.onOk()
            self.assertTrue(eow.result)
            self.assertEqual(_c.call_count, 1)

    def test_initUI(self):
        """test initUI"""
        with patch(
            "physbiblio.gui.commonClasses.EditObjectWindow.initUI", autospec=True
        ) as _iu:
            eow = EditObjectWindow()
        self.assertEqual(eow.currGrid, None)
        eow = EditObjectWindow()
        self.assertIsInstance(eow.currGrid, QGridLayout)
        self.assertEqual(eow.layout(), eow.currGrid)
        self.assertEqual(eow.currGrid.spacing(), 1)


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestPBThread(GUITestCase):
    """Test the PBThread class"""

    def test_methods(self):
        """test all the methods in the class"""
        with patch("PySide2.QtCore.QThread.__init__", autospec=True) as _in:
            PBThread()
            self.assertEqual(_in.call_count, 1)
        mt = PBThread()
        self.assertIsInstance(mt, QThread)
        self.assertRaises(NotImplementedError, lambda: mt.run())
        self.assertRaises(NotImplementedError, lambda: mt.setStopFlag())
        self.assertIsInstance(mt.finished, Signal)
        with patch("time.sleep", autospec=True) as _sl, patch(
            "PySide2.QtCore.QThread.start", autospec=True
        ) as _st:
            mt.start()
            _sl.assert_called_once_with(0.3)
            _st.assert_called_once_with(mt)
            _st.reset_mock()
            mt.start(1, a="try")
            _st.assert_called_once_with(mt, 1, a="try")


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestWriteStream(GUITestCase):
    """Test the WriteStream class"""

    def test_methods(self):
        """Test __init__, write, run methods"""
        queue = Queue()
        ws = WriteStream(queue)
        self.assertIsInstance(ws, PBThread)
        self.assertEqual(ws.queue, queue)
        self.assertTrue(ws.running)
        self.assertEqual(ws.parent(), None)
        self.assertIsInstance(ws.newText, Signal)
        self.assertIsInstance(ws.finished, Signal)
        ew = QWidget()
        ws = WriteStream(queue, parent=ew)
        self.assertEqual(ws.parent(), ew)
        ws.finished.connect(lambda: fakeExec_writeStream_fin(ws))
        if sys.version_info[0] < 3:
            with patch("Queue.Queue.put", autospec=True) as _put:
                ws.write("abc")
                _put.assert_called_once_with(queue, "abc")
            with patch("Queue.Queue.get", return_value="abc", autospec=True) as _get:
                ws.newText.connect(lambda x: fakeExec_writeStream_mys(ws, x))
                ws.run()
                self.assertEqual(_get.call_count, 1)
                self.assertEqual(ws.text, "abc")
                self.assertTrue(ws.fin)
        else:
            with patch("queue.Queue.put", autospec=True) as _put:
                ws.write("abc")
                _put.assert_called_once_with(queue, "abc")
            with patch("queue.Queue.get", return_value="abc", autospec=True) as _get:
                ws.newText.connect(lambda x: fakeExec_writeStream_mys(ws, x))
                ws.run()
                self.assertEqual(_get.call_count, 1)
                self.assertEqual(ws.text, "abc")
                self.assertTrue(ws.fin)


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestPBTableView(GUITestCase):
    """Test the PBTableView class"""

    def test_methods(self):
        """Test __init__, contextMenuEvent methods"""
        p = ObjListWindow()
        mtw = PBTableView(p)
        self.assertIsInstance(mtw, QTableView)
        self.assertEqual(mtw.parent(), p)
        e = QContextMenuEvent(QContextMenuEvent.Mouse, QPoint())
        with patch(
            "PySide2.QtGui.QContextMenuEvent.x", return_value=12, autospec=True
        ) as _x, patch(
            "PySide2.QtGui.QContextMenuEvent.y", return_value=24, autospec=True
        ) as _y, patch(
            "PySide2.QtWidgets.QTableView.rowAt", return_value=0, autospec=True
        ) as _r, patch(
            "PySide2.QtWidgets.QTableView.columnAt", return_value=1, autospec=True
        ) as _c, patch(
            "physbiblio.gui.commonClasses.ObjListWindow.triggeredContextMenuEvent",
            autospec=True,
        ) as _t:
            mtw.contextMenuEvent(e)
            _x.assert_called_once_with()
            _y.assert_called_once_with()
            _r.assert_called_once_with(24)
            _c.assert_called_once_with(12)
            _t.assert_called_once_with(p, 0, 1, e)


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestPBTableModel(GUITestCase):
    """Test the PBTableModel class"""

    def test_init(self):
        """test constructor"""
        p = ObjListWindow()
        mtm = PBTableModel(p, ["a", "b"])
        self.assertIsInstance(mtm, QAbstractTableModel)
        self.assertEqual(mtm.header, ["a", "b"])
        self.assertEqual(mtm.parentObj, p)
        self.assertEqual(mtm.previous, [])
        self.assertFalse(mtm.ask)
        mtm = PBTableModel(p, ["a", "b"], ask=True, previous=[1])
        self.assertEqual(mtm.previous, [1])
        self.assertTrue(mtm.ask)

    def test_changeAsk(self):
        """Test changeAsk"""

        def setAbout(cl):
            cl.about = True

        def setChanged(cl):
            cl.changed = True

        p = ObjListWindow()
        mtm = PBTableModel(p, ["a", "b"])
        mtm.about = False
        mtm.changed = False
        mtm.layoutAboutToBeChanged.connect(lambda: setAbout(mtm))
        mtm.layoutChanged.connect(lambda: setChanged(mtm))
        self.assertFalse(mtm.about)
        self.assertFalse(mtm.changed)
        mtm.changeAsk()
        self.assertTrue(mtm.about)
        self.assertTrue(mtm.changed)
        self.assertTrue(mtm.ask)
        mtm.changeAsk()
        self.assertFalse(mtm.ask)
        mtm.changeAsk(False)
        self.assertFalse(mtm.ask)
        mtm.changeAsk(True)
        self.assertTrue(mtm.ask)
        mtm.changeAsk("a")  # does nothing
        self.assertTrue(mtm.ask)

    def test_getIdentifier(self):
        """Test that getIdentifier is not implemented"""
        p = ObjListWindow()
        mtm = PBTableModel(p, ["a", "b"])
        self.assertRaises(NotImplementedError, lambda: mtm.getIdentifier("a"))

    def test_prepareSelected(self):
        """test prepareSelected"""

        def setAbout(cl):
            cl.about = True

        def setChanged(cl):
            cl.changed = True

        p = ObjListWindow()
        mtm = PBTableModel(p, ["a", "b"])
        mtm.dataList = [{"a": 1, "b": 3}, {"a": 2, "b": 2}, {"a": 3, "b": 1}]
        mtm.about = False
        mtm.changed = False
        mtm.layoutAboutToBeChanged.connect(lambda: setAbout(mtm))
        mtm.layoutChanged.connect(lambda: setChanged(mtm))
        self.assertFalse(mtm.about)
        self.assertFalse(mtm.changed)
        with patch(
            "physbiblio.gui.commonClasses.PBTableModel.getIdentifier",
            side_effect=[0, 1, 2],
            autospec=True,
        ) as _gi:
            mtm.prepareSelected()
            _gi.assert_has_calls(
                [
                    call(mtm, {"a": 1, "b": 3}),
                    call(mtm, {"a": 2, "b": 2}),
                    call(mtm, {"a": 3, "b": 1}),
                ]
            )
        self.assertEqual(mtm.selectedElements, {0: False, 1: False, 2: False})
        mtm = PBTableModel(p, ["a", "b"], previous=[2])
        mtm.dataList = [{"a": 1, "b": 3}, {"a": 2, "b": 2}, {"a": 3, "b": 1}]
        with patch(
            "physbiblio.gui.commonClasses.PBTableModel.getIdentifier",
            side_effect=[0, 1, 2],
            autospec=True,
        ) as _gi:
            mtm.prepareSelected()
            _gi.assert_has_calls(
                [
                    call(mtm, {"a": 1, "b": 3}),
                    call(mtm, {"a": 2, "b": 2}),
                    call(mtm, {"a": 3, "b": 1}),
                ]
            )
        self.assertEqual(mtm.selectedElements, {0: False, 1: False, 2: True})
        mtm = PBTableModel(p, ["a", "b"], previous=[1, 5])
        mtm.dataList = [{"a": 1, "b": 3}, {"a": 2, "b": 2}, {"a": 3, "b": 1}]
        with patch(
            "physbiblio.gui.commonClasses.PBTableModel.getIdentifier",
            side_effect=[0, 1, 2],
            autospec=True,
        ) as _gi, patch("logging.Logger.exception") as _ex:
            mtm.prepareSelected()
            _ex.assert_called_once_with("Invalid identifier in previous selection: 5")
            _gi.assert_has_calls(
                [
                    call(mtm, {"a": 1, "b": 3}),
                    call(mtm, {"a": 2, "b": 2}),
                    call(mtm, {"a": 3, "b": 1}),
                ]
            )
        self.assertEqual(mtm.selectedElements, {0: False, 1: True, 2: False})
        mtm = PBTableModel(p, ["a", "b"])
        with patch(
            "physbiblio.gui.commonClasses.PBTableModel.getIdentifier",
            side_effect=[0, 1, 2],
            autospec=True,
        ) as _gi, patch("logging.Logger.exception") as _ex:
            mtm.prepareSelected()
            _ex.assert_called_once_with("dataList is not defined!")
            self.assertEqual(_gi.call_count, 0)
        self.assertEqual(mtm.selectedElements, {})

    def test_un_selectAll(self):
        """test selectAll and unselectAll"""

        def setAbout(cl):
            cl.about = True

        def setChanged(cl):
            cl.changed = True

        p = ObjListWindow()
        mtm = PBTableModel(p, ["a", "b"])
        dl = [{"a": 1, "b": 3}, {"a": 2, "b": 2}, {"a": 3, "b": 1}]
        mtm.dataList = dl
        mtm.about = False
        mtm.changed = False
        mtm.layoutAboutToBeChanged.connect(lambda: setAbout(mtm))
        mtm.layoutChanged.connect(lambda: setChanged(mtm))
        self.assertFalse(mtm.about)
        self.assertFalse(mtm.changed)
        mtm.selectedElements = {0: False, 1: False, 2: False}
        mtm.selectAll()
        self.assertTrue(mtm.about)
        self.assertTrue(mtm.changed)
        self.assertEqual(mtm.selectedElements, {0: True, 1: True, 2: True})
        mtm.about = False
        mtm.changed = False
        mtm.unselectAll()
        self.assertTrue(mtm.about)
        self.assertTrue(mtm.changed)
        self.assertEqual(mtm.selectedElements, {0: False, 1: False, 2: False})

    def test_addImage(self):
        """Test addImage"""
        p = ObjListWindow()
        mtm = PBTableModel(p, ["a", "b"])
        img = mtm.addImage(":/images/edit.png", 51)
        self.assertIsInstance(img, QPixmap)
        self.assertEqual(img.height(), 51)
        img = mtm.addImage(":/images/nonexistent", 51)
        self.assertTrue(img.isNull())

    def test_addImages(self):
        """Test addImages"""
        p = ObjListWindow()
        mtm = PBTableModel(p, ["a", "b"])
        qp = mtm.addImages([":/images/edit.png", ":/images/find.png"], 31)
        self.assertIsInstance(qp, QPixmap)
        self.assertEqual(qp.height(), 31)
        basepixm = QPixmap(96, 48)
        qpixm = QPixmap(":/images/edit.png")
        painter = QPainter(basepixm)
        with patch(
            "physbiblio.gui.commonClasses.QPixmap",
            side_effect=[basepixm, qpixm, qpixm],
            autospec=True,
        ) as _qpm, patch("PySide2.QtGui.QPixmap.fill", autospec=True) as _fi, patch(
            "physbiblio.gui.commonClasses.QPainter", return_value=painter, autospec=True
        ) as _qpai, patch(
            "PySide2.QtGui.QPainter.drawPixmap", autospec=True
        ) as _drp:
            qp = mtm.addImages([":/images/edit.png", ":/images/find.png"], 31)
            _qpm.assert_has_calls(
                [call(96, 48), call(":/images/edit.png"), call(":/images/find.png")]
            )
            _fi.assert_called_once_with(Qt.transparent)
            _qpai.assert_called_once_with(basepixm)
            _drp.assert_has_calls([call(0, 0, qpixm), call(48, 0, qpixm)])
        self.assertFalse(mtm.painter.isActive())

    def test_rowCount(self):
        """Test columnCount"""
        p = ObjListWindow()
        mtm = PBTableModel(p, ["a", "b"])
        self.assertEqual(mtm.rowCount(), 0)
        mtm.dataList = [{"a": 1, "b": 3}, {"a": 2, "b": 2}, {"a": 3, "b": 1}]
        self.assertEqual(mtm.rowCount(), 3)
        mtm = PBTableModel(p, 1)
        self.assertEqual(mtm.rowCount(), 0)

    def test_columnCount(self):
        """Test columnCount"""
        p = ObjListWindow()
        mtm = PBTableModel(p, ["a", "b"])
        self.assertEqual(mtm.columnCount(), 2)
        self.assertEqual(mtm.columnCount(p), 2)
        mtm = PBTableModel(p, 1)
        self.assertEqual(mtm.columnCount(), 0)

    def test_data(self):
        """Test that data is not implemented"""
        p = ObjListWindow()
        mtm = PBTableModel(p, ["a", "b"])
        self.assertRaises(NotImplementedError, lambda: mtm.data(1, 1))

    def test_flags(self):
        """test flags"""
        idx = QModelIndex()
        p = ObjListWindow()
        mtm = PBTableModel(p, ["a", "b"])
        self.assertEqual(mtm.flags(idx), Qt.NoItemFlags)
        with patch("PySide2.QtCore.QModelIndex.isValid", return_value=True) as _iv:
            with patch(
                "PySide2.QtCore.QModelIndex.column", return_value=1, autospec=True
            ) as _c:
                self.assertEqual(mtm.flags(idx), Qt.ItemIsEnabled | Qt.ItemIsSelectable)
            with patch(
                "PySide2.QtCore.QModelIndex.column", return_value=0, autospec=True
            ) as _c:
                self.assertEqual(mtm.flags(idx), Qt.ItemIsEnabled | Qt.ItemIsSelectable)
            mtm.ask = True
            with patch(
                "PySide2.QtCore.QModelIndex.column", return_value=0, autospec=True
            ) as _c:
                self.assertEqual(
                    mtm.flags(idx),
                    Qt.ItemIsUserCheckable
                    | Qt.ItemIsEditable
                    | Qt.ItemIsEnabled
                    | Qt.ItemIsSelectable,
                )

    def test_headerData(self):
        """test headerData"""
        p = ObjListWindow()
        mtm = PBTableModel(p, ["a", "b"])
        self.assertEqual(mtm.headerData(0, Qt.Vertical, Qt.DisplayRole), None)
        self.assertEqual(mtm.headerData(0, Qt.Horizontal, Qt.EditRole), None)
        self.assertEqual(mtm.headerData(0, Qt.Horizontal, Qt.DisplayRole), "a")

    def test_setData(self):
        """Test that setData is not implemented"""
        p = ObjListWindow()
        mtm = PBTableModel(p, ["a", "b"])
        self.assertRaises(NotImplementedError, lambda: mtm.setData(1, 1, 1))


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestTreeNode(GUITestCase):
    """Test the TreeNode class"""

    def test_methods(self):
        """test the __init__ and _getChildren methods"""
        ew = QWidget()
        with patch(
            "physbiblio.gui.commonClasses.TreeNode._getChildren",
            return_value="def",
            autospec=True,
        ) as _gc:
            tn = TreeNode(ew, "abc")
            self.assertEqual(tn.parent(), ew)
            self.assertEqual(tn.parentObj, ew)
            self.assertEqual(tn.row, "abc")
            self.assertEqual(tn.subnodes, "def")
            _gc.assert_called_once_with(tn)
        self.assertRaises(NotImplementedError, lambda: tn._getChildren())


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestTreeModel(GUITestCase):
    """Test the TreeModel class"""

    def createTm(self):
        """Create a model structure for tests"""
        with patch(
            "physbiblio.gui.commonClasses.TreeModel._getRootNodes", autospec=True
        ):
            tm = TreeModel()
        tm.rootElements = [
            NamedElement(
                0,
                "main",
                [
                    NamedElement(1, "test1", [NamedElement(2, "test2", [])]),
                    NamedElement(3, "test3", []),
                ],
            )
        ]
        tm.rootNodes = [NamedNode(tm.rootElements[0], None, 0)]
        return tm

    def test_init(self):
        """test the constructor"""
        with patch(
            "physbiblio.gui.commonClasses.TreeModel._getRootNodes",
            return_value="empty",
            autospec=True,
        ) as _gr:
            tm = TreeModel()
            self.assertEqual(_gr.call_count, 1)
        self.assertIsInstance(tm, QAbstractItemModel)
        self.assertEqual(tm.rootNodes, "empty")

    def test_getRootNodes(self):
        """test the non implemented _getRootNodes"""
        self.assertRaises(NotImplementedError, lambda: TreeModel())
        with patch(
            "physbiblio.gui.commonClasses.TreeModel._getRootNodes",
            return_value="empty",
            autospec=True,
        ) as _gr:
            tm = TreeModel()
        self.assertRaises(NotImplementedError, lambda: tm._getRootNodes())

    def test_index(self):
        """Test the function that returns the object index"""
        tm = self.createTm()
        with patch("logging.Logger.debug") as _d:
            qmi = tm.index(0, 0, None)
            self.assertIsInstance(qmi, QModelIndex)
            self.assertFalse(qmi.isValid())
            _d.assert_called_once_with(
                "Invalid parent 'None' in TreeModel.index", exc_info=True
            )
        qmi = tm.index(0, 0)
        self.assertIsInstance(qmi, QModelIndex)
        self.assertTrue(qmi.isValid())
        self.assertEqual(TreeNode.cast(qmi).element.name, "main")
        qmi = tm.index(1, 0)
        self.assertIsInstance(qmi, QModelIndex)
        self.assertFalse(qmi.isValid())
        p = tm.index(0, 0)
        qmi = tm.index(0, 0, parent=p)
        self.assertIsInstance(qmi, QModelIndex)
        self.assertTrue(qmi.isValid())
        self.assertEqual(TreeNode.cast(qmi).element.name, "test1")
        qmi = tm.index(1, 0, parent=p)
        self.assertIsInstance(qmi, QModelIndex)
        self.assertTrue(qmi.isValid())
        self.assertEqual(TreeNode.cast(qmi).element.name, "test3")
        qmi = tm.index(2, 0, parent=p)
        self.assertIsInstance(qmi, QModelIndex)
        self.assertFalse(qmi.isValid())
        qmi = tm.index(0, 0, parent=tm.index(1, 0, parent=p))
        self.assertIsInstance(qmi, QModelIndex)
        self.assertFalse(qmi.isValid())
        qmi = tm.index(0, 0, parent=tm.index(0, 0, parent=p))
        self.assertIsInstance(qmi, QModelIndex)
        self.assertTrue(qmi.isValid())
        self.assertEqual(TreeNode.cast(qmi).element.name, "test2")
        qmi = tm.index(0, 0, parent=tm.index(5, 0))
        self.assertIsInstance(qmi, QModelIndex)
        self.assertTrue(qmi.isValid())
        self.assertEqual(TreeNode.cast(qmi).element.name, "main")
        qmi = tm.index(3, 0, parent=tm.index(5, 0))
        self.assertIsInstance(qmi, QModelIndex)
        self.assertFalse(qmi.isValid())

    def test_parent(self):
        """Test the function that returns the parent object"""
        tm = self.createTm()
        mn = tm.index(0, 0)
        t1 = tm.index(0, 0, parent=mn)
        t2 = tm.index(0, 0, parent=t1)
        t3 = tm.index(1, 0, parent=mn)

        with patch("logging.Logger.debug") as _d:
            qmi = tm.parent(None)
            self.assertIsInstance(qmi, QModelIndex)
            self.assertFalse(qmi.isValid())
            _d.assert_called_once_with(
                "Invalid index 'None' in TreeModel.parent", exc_info=True
            )
        qmi = tm.parent(tm.index(3, 0))
        self.assertIsInstance(qmi, QModelIndex)
        self.assertFalse(qmi.isValid())

        qmi = tm.parent(mn)
        self.assertIsInstance(qmi, QModelIndex)
        self.assertFalse(qmi.isValid())
        qmi = tm.parent(t1)
        self.assertIsInstance(qmi, QModelIndex)
        self.assertTrue(qmi.isValid())
        self.assertEqual(TreeNode.cast(qmi).element.name, "main")
        qmi = tm.parent(t3)
        self.assertIsInstance(qmi, QModelIndex)
        self.assertTrue(qmi.isValid())
        self.assertEqual(TreeNode.cast(qmi).element.name, "main")
        qmi = tm.parent(t2)
        self.assertIsInstance(qmi, QModelIndex)
        self.assertTrue(qmi.isValid())
        self.assertEqual(TreeNode.cast(qmi).element.name, "test1")

    def test_rowCount(self):
        """test the counting of rows"""
        tm = self.createTm()
        mn = tm.index(0, 0)
        t1 = tm.index(0, 0, parent=mn)
        t2 = tm.index(0, 0, parent=t1)
        t3 = tm.index(1, 0, parent=mn)
        with patch("logging.Logger.debug") as _d:
            rc = tm.rowCount(None)
            self.assertEqual(rc, 1)
            _d.assert_called_once_with(
                "Invalid parent 'None' in TreeModel.rowCount", exc_info=True
            )
        self.assertEqual(tm.rowCount(), 1)
        self.assertEqual(tm.rowCount(tm.index(5, 0)), 1)
        self.assertEqual(tm.rowCount(mn), 2)
        self.assertEqual(tm.rowCount(t1), 1)
        self.assertEqual(tm.rowCount(t2), 0)
        self.assertEqual(tm.rowCount(t3), 0)


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestNamedElement(GUITestCase):
    """Test the NamedElement class"""

    def test_init(self):
        """test the constructor"""
        with patch(
            "physbiblio.gui.commonClasses.catString",
            return_value="abcde",
            autospec=True,
        ) as _cs:
            ne = NamedElement(0, "main", ["a", "b"])
            _cs.assert_called_once_with(0, pBDB)
        self.assertEqual(ne.idCat, 0)
        self.assertEqual(ne.name, "main")
        self.assertEqual(ne.text, "abcde")
        self.assertEqual(ne.subelements, ["a", "b"])


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestNamedNode(GUITestCase):
    """Test the NamedNode class"""

    def test_methods(self):
        """test __init__ and _getChildren"""
        el = NamedElement(1, "tags", [])
        main = NamedElement(0, "main", [el])
        nn = NamedNode(main, None, 0)
        self.assertIsInstance(nn, TreeNode)
        with patch(
            "physbiblio.gui.commonClasses.TreeNode.__init__",
            return_value=None,
            autospec=True,
        ) as _in:
            nn = NamedNode(main, None, 0)
            _in.assert_called_once_with(nn, None, 0)
        self.assertEqual(nn.element, main)
        with patch(
            "physbiblio.gui.commonClasses.NamedNode.__init__",
            return_value=None,
            autospec=True,
        ) as _in:
            res = nn._getChildren()
            _in.assert_called_once_with(res[0], el, nn, 0)
        self.assertIsInstance(res, list)
        self.assertEqual(len(res), 1)
        self.assertIsInstance(res[0], NamedNode)


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestLeafFilterProxyModel(GUITestCase):
    """Test the LeafFilterProxyModel class"""

    def test_init(self):
        """just check that the created object is an instance of parent class"""
        lf = LeafFilterProxyModel()
        self.assertIsInstance(lf, QSortFilterProxyModel)

    def test_filterAcceptsRow(self):
        """Test filterAcceptsRow with all possible combinations"""
        lf = LeafFilterProxyModel()
        with patch(
            "physbiblio.gui.commonClasses.LeafFilterProxyModel."
            + "filterAcceptsRowItself",
            return_value=True,
            autospec=True,
        ) as _its, patch(
            "physbiblio.gui.commonClasses.LeafFilterProxyModel."
            + "filterAcceptsAnyParent",
            autospec=True,
        ) as _par, patch(
            "physbiblio.gui.commonClasses.LeafFilterProxyModel."
            + "hasAcceptedChildren",
            autospec=True,
        ) as _chi:
            self.assertTrue(lf.filterAcceptsRow(0, None))
            _its.assert_called_once_with(lf, 0, None)
            self.assertEqual(_par.call_count, 0)
            self.assertEqual(_chi.call_count, 0)
        with patch(
            "physbiblio.gui.commonClasses.LeafFilterProxyModel."
            + "filterAcceptsRowItself",
            return_value=False,
            autospec=True,
        ) as _its, patch(
            "physbiblio.gui.commonClasses.LeafFilterProxyModel."
            + "filterAcceptsAnyParent",
            return_value=True,
            autospec=True,
        ) as _par, patch(
            "physbiblio.gui.commonClasses.LeafFilterProxyModel."
            + "hasAcceptedChildren",
            autospec=True,
        ) as _chi:
            self.assertTrue(lf.filterAcceptsRow(0, None))
            _its.assert_called_once_with(lf, 0, None)
            _par.assert_called_once_with(lf, None)
            self.assertEqual(_chi.call_count, 0)
        with patch(
            "physbiblio.gui.commonClasses.LeafFilterProxyModel."
            + "filterAcceptsRowItself",
            return_value=False,
            autospec=True,
        ) as _its, patch(
            "physbiblio.gui.commonClasses.LeafFilterProxyModel."
            + "filterAcceptsAnyParent",
            return_value=False,
            autospec=True,
        ) as _par, patch(
            "physbiblio.gui.commonClasses.LeafFilterProxyModel."
            + "hasAcceptedChildren",
            return_value=True,
            autospec=True,
        ) as _chi:
            self.assertTrue(lf.filterAcceptsRow(0, None))
            _its.assert_called_once_with(lf, 0, None)
            _par.assert_called_once_with(lf, None)
            _chi.assert_called_once_with(lf, 0, None)
        with patch(
            "physbiblio.gui.commonClasses.LeafFilterProxyModel."
            + "filterAcceptsRowItself",
            return_value=False,
            autospec=True,
        ) as _its, patch(
            "physbiblio.gui.commonClasses.LeafFilterProxyModel."
            + "filterAcceptsAnyParent",
            return_value=False,
            autospec=True,
        ) as _par, patch(
            "physbiblio.gui.commonClasses.LeafFilterProxyModel."
            + "hasAcceptedChildren",
            return_value=False,
            autospec=True,
        ) as _chi:
            self.assertFalse(lf.filterAcceptsRow(0, None))
            _its.assert_called_once_with(lf, 0, None)
            _par.assert_called_once_with(lf, None)
            _chi.assert_called_once_with(lf, 0, None)

    def test_filterAcceptRowItself(self):
        """test filterAcceptRowItself"""
        lf = LeafFilterProxyModel()
        with patch(
            "PySide2.QtCore.QSortFilterProxyModel.filterAcceptsRow",
            return_value="testval",
            autospec=True,
        ) as _far:
            self.assertEqual(lf.filterAcceptsRowItself(0, "par"), "testval")
            _far.assert_called_once_with(0, "par")

    def test_filterAcceptsAnyParent(self):
        """test filterAcceptsAnyParent"""
        el = NamedElement(1, "tags", [])
        ot = NamedElement(2, "other", [])
        main = NamedElement(0, "main", [el])
        lf = LeafFilterProxyModel()
        tm = EmptyTreeModel([main, ot])
        lf.setSourceModel(tm)
        self.assertTrue(lf.filterAcceptsAnyParent(lf.index(0, 0, QModelIndex())))
        self.assertTrue(lf.filterAcceptsAnyParent(lf.index(1, 0, QModelIndex())))
        self.assertFalse(lf.filterAcceptsAnyParent(lf.index(2, 0, QModelIndex())))
        with patch(
            "PySide2.QtCore.QModelIndex.isValid",
            side_effect=[True, True, False],
            autospec=True,
        ) as _iv, patch(
            "physbiblio.gui.commonClasses.LeafFilterProxyModel."
            + "filterAcceptsRowItself",
            return_value=False,
            autospec=True,
        ) as _its:
            self.assertFalse(lf.filterAcceptsAnyParent(lf.index(0, 0, QModelIndex())))
            self.assertEqual(_iv.call_count, 3)
            self.assertEqual(_its.call_count, 2)
        with patch(
            "PySide2.QtCore.QModelIndex.isValid",
            side_effect=[True, True, False],
            autospec=True,
        ) as _iv, patch(
            "physbiblio.gui.commonClasses.LeafFilterProxyModel."
            + "filterAcceptsRowItself",
            return_value=False,
            autospec=True,
        ) as _its, patch(
            "physbiblio.gui.commonClasses.QModelIndex.parent",
            return_value=QModelIndex(),
            autospec=True,
        ) as _pa:
            self.assertFalse(lf.filterAcceptsAnyParent(lf.index(0, 0, QModelIndex())))
            self.assertEqual(_iv.call_count, 3)
            self.assertEqual(_its.call_count, 2)
            self.assertEqual(_pa.call_count, 4)
        with patch(
            "PySide2.QtCore.QModelIndex.isValid",
            side_effect=[True, True, False],
            autospec=True,
        ) as _iv, patch(
            "physbiblio.gui.commonClasses.LeafFilterProxyModel."
            + "filterAcceptsRowItself",
            return_value=True,
            autospec=True,
        ) as _its:
            self.assertTrue(lf.filterAcceptsAnyParent(lf.index(0, 0, QModelIndex())))
            self.assertEqual(_iv.call_count, 1)
            self.assertEqual(_its.call_count, 1)
        with patch(
            "PySide2.QtCore.QModelIndex.isValid",
            side_effect=[True, True, False],
            autospec=True,
        ) as _iv, patch(
            "physbiblio.gui.commonClasses.LeafFilterProxyModel."
            + "filterAcceptsRowItself",
            return_value=True,
            autospec=True,
        ) as _its, patch(
            "PySide2.QtCore.QModelIndex.parent", autospec=True
        ) as _pa:
            self.assertTrue(lf.filterAcceptsAnyParent(lf.index(0, 0, QModelIndex())))
            self.assertEqual(_iv.call_count, 1)
            self.assertEqual(_its.call_count, 1)
            self.assertEqual(_pa.call_count, 1)

    def test_hasAcceptedChildren(self):  # probably could be improved a bit
        """test hasAcceptedChildren"""
        el = NamedElement(1, "tags", [])
        main = NamedElement(0, "main", [el])
        lf = LeafFilterProxyModel()
        tm = EmptyTreeModel([main])
        lf.setSourceModel(tm)
        self.assertTrue(lf.hasAcceptedChildren(0, QModelIndex()))
        with patch(
            "physbiblio.gui.tests.test_commonClasses.EmptyTreeModel.rowCount",
            return_value=1,
            autospec=True,
        ) as _rc:
            self.assertTrue(lf.hasAcceptedChildren(1, QModelIndex()))
        with patch(
            "physbiblio.gui.commonClasses.LeafFilterProxyModel.filterAcceptsRow",
            return_value=True,
            autospec=True,
        ) as _acc:
            self.assertTrue(lf.hasAcceptedChildren(5, QModelIndex()))
            self.assertEqual(_acc.call_count, 1)
        with patch(
            "physbiblio.gui.commonClasses.LeafFilterProxyModel.filterAcceptsRow",
            return_value=False,
            autospec=True,
        ) as _acc, patch(
            "physbiblio.gui.tests.test_commonClasses.EmptyTreeModel.rowCount",
            return_value=1,
            autospec=True,
        ) as _rc:
            self.assertFalse(lf.hasAcceptedChildren(1, QModelIndex()))


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestPBDDTableWidget(GUITestCase):
    """Test the PBDDTableWidget class"""

    def test_init(self):
        """test init"""
        p = QWidget()
        mddtw = PBDDTableWidget(p, "head")
        self.assertIsInstance(mddtw, QTableWidget)
        self.assertEqual(mddtw.parent(), p)
        self.assertEqual(mddtw.columnCount(), 1)
        self.assertTrue(mddtw.dragEnabled())
        self.assertTrue(mddtw.acceptDrops())
        self.assertFalse(mddtw.dragDropOverwriteMode())
        self.assertEqual(mddtw.selectionBehavior(), QAbstractItemView.SelectRows)
        with patch(
            "physbiblio.gui.commonClasses.QTableWidget.setHorizontalHeaderLabels",
            autospec=True,
        ) as _shl:
            mddtw = PBDDTableWidget(p, "head")
            _shl.assert_called_once_with(["head"])

    def test_dropMimeData(self):
        """test dropMimeData"""
        p = QWidget()
        mddtw = PBDDTableWidget(p, "head")
        self.assertTrue(mddtw.dropMimeData(12, 0, None, None))
        self.assertEqual(mddtw.last_drop_row, 12)

    def test_dropEvent(self):
        """test dropEvent"""
        p = QWidget()
        sender = PBDDTableWidget(p, "head")
        item = QTableWidgetItem("source1")
        sender.insertRow(0)
        sender.setItem(0, 0, item)
        item = QTableWidgetItem("source2")
        sender.insertRow(1)
        sender.setItem(1, 0, item)
        sender.selectionModel().select(
            sender.model().index(0, 0), QItemSelectionModel.Select
        )

        mddtw = PBDDTableWidget(p, "head")
        item = QTableWidgetItem("test1")
        mddtw.insertRow(0)
        mddtw.setItem(0, 0, item)
        item = QTableWidgetItem("test2")
        mddtw.insertRow(1)
        mddtw.setItem(1, 0, item)

        mimedata = QMimeData()
        if sys.version_info[0] < 3:
            mimedata.setData(
                "application/x-qabstractitemmodeldatalist", QByteArray("source1")
            )
        else:
            mimedata.setData(
                "application/x-qabstractitemmodeldatalist", QByteArray(b"source1")
            )
        ev = QDropEvent(
            QPointF(1, 1),
            Qt.DropActions(Qt.MoveAction),
            mimedata,
            Qt.MouseButtons(Qt.LeftButton),
            Qt.KeyboardModifiers(Qt.NoModifier),
        )
        with patch(
            "PySide2.QtGui.QDropEvent.source", return_value=sender, autospec=True
        ) as _s:
            mddtw.dropEvent(ev)
            _s.assert_called_once_with()
        self.assertEqual(sender.rowCount(), 1)
        self.assertEqual(mddtw.rowCount(), 3)
        self.assertEqual(sender.item(0, 0).text(), "source2")
        self.assertEqual(mddtw.item(0, 0).text(), "source1")
        self.assertEqual(mddtw.item(1, 0).text(), "test1")
        self.assertEqual(mddtw.item(2, 0).text(), "test2")

        mddtw.selectionModel().select(
            mddtw.model().index(1, 0), QItemSelectionModel.Select
        )
        if sys.version_info[0] < 3:
            mimedata.setData(
                "application/x-qabstractitemmodeldatalist", QByteArray("test1")
            )
        else:
            mimedata.setData(
                "application/x-qabstractitemmodeldatalist", QByteArray(b"test1")
            )
        ev = QDropEvent(
            QPointF(1, 1),
            Qt.DropActions(Qt.MoveAction),
            mimedata,
            Qt.MouseButtons(Qt.LeftButton),
            Qt.KeyboardModifiers(Qt.NoModifier),
        )
        with patch(
            "PySide2.QtGui.QDropEvent.source", return_value=mddtw, autospec=True
        ) as _s:
            mddtw.dropEvent(ev)
            _s.assert_called_once_with()
        self.assertEqual(sender.rowCount(), 1)
        self.assertEqual(mddtw.rowCount(), 3)
        self.assertEqual(sender.item(0, 0).text(), "source2")
        self.assertEqual(mddtw.item(0, 0).text(), "test1")
        self.assertEqual(mddtw.item(1, 0).text(), "source1")
        self.assertEqual(mddtw.item(2, 0).text(), "test2")

    def test_getselectedRowsFast(self):
        """test getselectedRowsFast"""
        p = QWidget()
        mddtw = PBDDTableWidget(p, "head")
        a = mddtw.getselectedRowsFast()
        self.assertEqual(a, [])
        with patch(
            "PySide2.QtWidgets.QTableWidget.selectedItems",
            return_value=[QTableWidgetItem() for i in range(5)],
            autospec=True,
        ) as _si, patch(
            "PySide2.QtWidgets.QTableWidgetItem.row",
            side_effect=[1, 8, 3, 1, 2],
            autospec=True,
        ) as _r, patch(
            "PySide2.QtWidgets.QTableWidgetItem.text",
            side_effect=["de", "fg", "ab", "hi", "bibkey"],
            autospec=True,
        ) as _t:
            a = mddtw.getselectedRowsFast()
            self.assertEqual(a, [1, 3, 8])


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestPBMenu(GUITestCase):
    """Test the PBMenu class"""

    def test_init(self):
        """Test init"""
        mm = PBMenu()
        self.assertIsInstance(mm, QMenu)
        self.assertEqual(mm.possibleActions, [])
        self.assertFalse(mm.result)
        with patch("physbiblio.gui.commonClasses.QMenu.__init__", autospec=True) as _in:
            mm = PBMenu("abc")
            _in.assert_called_once_with("abc")

    def test_fillMenu(self):
        """test fillMenu"""
        mm = PBMenu()
        acts = [
            QAction("test1"),
            "abc",
            QAction("test2"),
            None,
            [
                "submenu",
                [QAction("Sub1"), QAction("Sub2"), ["subsub", [QAction("nest")]], None],
            ],
        ]
        mm.possibleActions = acts
        mm.fillMenu()
        macts = mm.actions()
        self.assertEqual(acts[0], macts[0])
        self.assertEqual(acts[2], macts[1])
        self.assertTrue(macts[2].isSeparator())
        self.assertEqual(macts[0].menu(), None)
        self.assertEqual(macts[1].menu(), None)
        self.assertEqual(macts[2].menu(), None)
        self.assertIsInstance(macts[3].menu(), QMenu)
        subm = macts[3].menu()
        self.assertEqual(subm.title(), acts[4][0])
        sacts = subm.actions()
        self.assertEqual(acts[4][1][0], sacts[0])
        self.assertEqual(acts[4][1][1], sacts[1])
        self.assertIsInstance(sacts[2].menu(), QMenu)
        subsub = sacts[2].menu()
        self.assertEqual(subsub.title(), acts[4][1][2][0])
        self.assertEqual(subsub.actions()[0], acts[4][1][2][1][0])
        self.assertTrue(sacts[3].isSeparator())

    def test_keyPressEvent(self):
        """test keyPressEvent"""
        mm = PBMenu()
        with patch("physbiblio.gui.commonClasses.QMenu.close", autospec=True) as _oc:
            QTest.keyPress(mm, "a")
            self.assertEqual(_oc.call_count, 0)
            QTest.keyPress(mm, Qt.Key_Enter)
            self.assertEqual(_oc.call_count, 0)
            QTest.keyPress(mm, Qt.Key_Escape)
            self.assertEqual(_oc.call_count, 1)


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestGuiViewEntry(GUITestCase):
    """Test the GuiViewEntry class"""

    def test_methods(self):
        """Test that the object is instance of `ViewEntry`
        and that openLink works
        """
        gve = GUIViewEntry()
        self.assertIsInstance(gve, ViewEntry)
        self.assertIsInstance(pBGuiView, GUIViewEntry)

        with patch(
            "PySide2.QtGui.QDesktopServices.openUrl", return_value=True
        ) as _ou, patch("logging.Logger.debug") as _db:
            gve.openLink(["abc", "def"], "link")
            _db.assert_has_calls(
                [
                    call("Opening link 'abc' for entry 'abc' successful!"),
                    call("Opening link 'def' for entry 'def' successful!"),
                ]
            )
            _ou.assert_has_calls([call(QUrl("abc")), call(QUrl("def"))])
        with patch(
            "PySide2.QtGui.QDesktopServices.openUrl", return_value=False
        ) as _ou, patch("logging.Logger.warning") as _db:
            gve.openLink(["abc", "def"], "link")
            _db.assert_has_calls(
                [
                    call("Opening link for 'abc' failed!"),
                    call("Opening link for 'def' failed!"),
                ]
            )
            _ou.assert_has_calls([call(QUrl("abc")), call(QUrl("def"))])
        with patch(
            "PySide2.QtGui.QDesktopServices.openUrl", return_value=True
        ) as _ou, patch(
            "physbiblio.view.ViewEntry.getLink", return_value="mylink", autospec=True
        ) as _gl:
            gve.openLink("abc", arg="somearg", fileArg="somefilearg")
            _gl.assert_called_once_with(
                gve, "abc", arg="somearg", fileArg="somefilearg"
            )
            _ou.assert_called_once_with(QUrl("mylink"))
        with patch(
            "PySide2.QtGui.QDesktopServices.openUrl", return_value=True
        ) as _ou, patch(
            "physbiblio.gui.commonClasses.QUrl.fromLocalFile",
            return_value=QUrl("mylink"),
        ) as _fl:
            gve.openLink("abc", arg="file", fileArg="/a/b/c")
            _fl.assert_called_once_with("/a/b/c")
            _ou.assert_called_once_with(QUrl("mylink"))


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestPBImportedTableModel(GUITestCase):
    """Test the PBImportedTableModel class"""

    def test_init(self):
        """test init"""
        with patch(
            "physbiblio.gui.commonClasses.PBTableModel.prepareSelected", autospec=True
        ) as _ps:
            mitm = PBImportedTableModel(
                QWidget(),
                {
                    "a": {"exist": False, "bibpars": {"ID": "a"}},
                    "b": {"exist": False, "bibpars": {"ID": "b"}},
                },
                ["key", "id"],
                "bibkey",
            )
            self.assertEqual(_ps.call_count, 1)
        self.assertIsInstance(mitm, PBTableModel)
        self.assertEqual(mitm.typeClass, "imports")
        self.assertEqual(mitm.idName, "bibkey")
        self.assertEqual(mitm.bibsOrder, ["a", "b"])
        self.assertEqual(mitm.dataList, [{"ID": "a"}, {"ID": "b"}])
        self.assertEqual(mitm.existList, [False, False])

    def test_getIdentifier(self):
        """test getIdentifier"""
        mitm = PBImportedTableModel(
            QWidget(),
            {
                "a": {"exist": False, "bibpars": {"ID": "a"}},
                "b": {"exist": False, "bibpars": {"ID": "b"}},
            },
            ["key", "id"],
        )
        self.assertEqual(mitm.getIdentifier({"id": "a", "bibkey": "b", "ID": "i"}), "i")

    def test_data(self):
        """test data"""
        mitm = PBImportedTableModel(
            QWidget(),
            {
                "a": {"exist": False, "bibpars": {"ID": "a", "key": "a"}},
                "b": {"exist": False, "bibpars": {"ID": "b", "key": "b"}},
            },
            ["key", "ID"],
        )
        with patch(
            "PySide2.QtCore.QModelIndex.isValid", return_value=False, autospec=True
        ) as _iv:
            self.assertEqual(mitm.data(QModelIndex(), Qt.DisplayRole), None)
        with patch(
            "PySide2.QtCore.QModelIndex.isValid", return_value=True, autospec=True
        ) as _iv:
            with patch(
                "PySide2.QtCore.QModelIndex.column", return_value=10, autospec=True
            ) as _c, patch(
                "PySide2.QtCore.QModelIndex.row", return_value=10, autospec=True
            ) as _r:
                self.assertEqual(mitm.data(QModelIndex(), Qt.DisplayRole), None)
            with patch(
                "PySide2.QtCore.QModelIndex.column", return_value=0, autospec=True
            ) as _c, patch(
                "PySide2.QtCore.QModelIndex.row", return_value=0, autospec=True
            ) as _r:
                self.assertEqual(
                    mitm.data(QModelIndex(), Qt.CheckStateRole), Qt.Unchecked
                )
                self.assertTrue(
                    mitm.setData(QModelIndex(), Qt.Checked, Qt.CheckStateRole)
                )
                self.assertEqual(
                    mitm.data(QModelIndex(), Qt.CheckStateRole), Qt.Checked
                )
            with patch(
                "PySide2.QtCore.QModelIndex.column", return_value=0, autospec=True
            ) as _c, patch(
                "PySide2.QtCore.QModelIndex.row", return_value=0, autospec=True
            ) as _r:
                mitm.existList[0] = True
                self.assertEqual(
                    mitm.data(QModelIndex(), Qt.EditRole), "a - already existing"
                )
                self.assertEqual(
                    mitm.data(QModelIndex(), Qt.DisplayRole), "a - already existing"
                )
                self.assertEqual(mitm.data(QModelIndex(), Qt.CheckStateRole), None)
            with patch(
                "PySide2.QtCore.QModelIndex.column", return_value=1, autospec=True
            ) as _c, patch(
                "PySide2.QtCore.QModelIndex.row", return_value=0, autospec=True
            ) as _r:
                self.assertEqual(mitm.data(QModelIndex(), Qt.EditRole), "a")
                self.assertEqual(mitm.data(QModelIndex(), Qt.DisplayRole), "a")
                self.assertEqual(mitm.data(QModelIndex(), Qt.CheckStateRole), None)

    def test_setData(self):
        """test setData"""
        mitm = PBImportedTableModel(
            QWidget(),
            {
                "a": {"exist": False, "bibpars": {"ID": "a"}},
                "b": {"exist": False, "bibpars": {"ID": "b"}},
            },
            ["key", "id"],
        )
        mitm.dataChanged.connect(fakeExec_dataChanged)
        ids = {"a": False, "b": False}
        with patch(
            "PySide2.QtCore.QModelIndex.isValid", return_value=True, autospec=True
        ) as _iv:
            self.assertTrue(mitm.setData(QModelIndex(), "abc", Qt.DisplayRole))
            self.assertEqual(ids, mitm.selectedElements)
            with patch(
                "PySide2.QtCore.QModelIndex.column", return_value=1, autospec=True
            ) as _c:
                self.assertTrue(mitm.setData(QModelIndex(), "abc", Qt.CheckStateRole))
                self.assertEqual(ids, mitm.selectedElements)
            with patch(
                "PySide2.QtCore.QModelIndex.column", return_value=0, autospec=True
            ) as _c, patch(
                "PySide2.QtCore.QModelIndex.row", return_value=0, autospec=True
            ) as _r:
                self.assertTrue(
                    mitm.setData(QModelIndex(), Qt.Checked, Qt.CheckStateRole)
                )
                self.assertEqual({"a": True, "b": False}, mitm.selectedElements)
                self.assertTrue(
                    mitm.setData(QModelIndex(), Qt.Unchecked, Qt.CheckStateRole)
                )
                self.assertEqual({"a": False, "b": False}, mitm.selectedElements)
                self.assertTrue(
                    mitm.setData(QModelIndex(), Qt.Checked, Qt.CheckStateRole)
                )
                self.assertEqual({"a": True, "b": False}, mitm.selectedElements)
                self.assertTrue(mitm.setData(QModelIndex(), "abc", Qt.CheckStateRole))
                self.assertEqual({"a": False, "b": False}, mitm.selectedElements)
        with patch(
            "PySide2.QtCore.QModelIndex.isValid", return_value=False, autospec=True
        ) as _iv:
            self.assertFalse(mitm.setData(QModelIndex(), Qt.Checked, Qt.DisplayRole))

    def test_flags(self):
        """test flags"""
        mitm = PBImportedTableModel(
            QWidget(),
            {
                "a": {"exist": False, "bibpars": {"ID": "a"}},
                "b": {"exist": True, "bibpars": {"ID": "b"}},
            },
            ["key", "id"],
        )
        with patch(
            "PySide2.QtCore.QModelIndex.isValid", return_value=False, autospec=True
        ) as _iv:
            self.assertEqual(mitm.flags(QModelIndex()), Qt.NoItemFlags)
        with patch(
            "PySide2.QtCore.QModelIndex.isValid", return_value=True, autospec=True
        ) as _iv, patch(
            "PySide2.QtCore.QModelIndex.column", return_value=1, autospec=True
        ) as _c, patch(
            "PySide2.QtCore.QModelIndex.row", side_effect=[0, 1], autospec=True
        ) as _r:
            self.assertEqual(
                mitm.flags(QModelIndex()), Qt.ItemIsEnabled | Qt.ItemIsSelectable
            )
            self.assertEqual(
                mitm.flags(QModelIndex()), Qt.ItemIsEnabled | Qt.ItemIsSelectable
            )
        with patch(
            "PySide2.QtCore.QModelIndex.isValid", return_value=True, autospec=True
        ) as _iv, patch(
            "PySide2.QtCore.QModelIndex.column", return_value=0, autospec=True
        ) as _c, patch(
            "PySide2.QtCore.QModelIndex.row", side_effect=[0, 1], autospec=True
        ) as _r:
            self.assertEqual(
                mitm.flags(QModelIndex()),
                Qt.ItemIsUserCheckable
                | Qt.ItemIsEditable
                | Qt.ItemIsEnabled
                | Qt.ItemIsSelectable,
            )
            self.assertEqual(
                mitm.flags(QModelIndex()), Qt.ItemIsEnabled | Qt.ItemIsSelectable
            )


@unittest.skipIf(skipTestsSettings.gui, "GUI tests")
class TestObjectWithSignal(GUITestCase):
    """Test the ObjectWithSignal class"""

    def test_class(self):
        """test the class inheritance and signal existence"""
        o = ObjectWithSignal()
        self.assertIsInstance(o, QObject)
        self.assertIsInstance(o.customSignal, Signal)


if __name__ == "__main__":
    unittest.main()
